package controller;

import java.io.File;
import java.util.ArrayList;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import model.Cassette;

public abstract class FileAction implements IOAction {

	// TODO remove this
	private static File defaultDir; // = new File("/home/mihai/Desktop/dp-samples");
	protected ArrayList<File> source = new ArrayList<File>();
	protected Cassette cassette;
	private File lastDir;
	protected FileFilter selectedFilter;

	@Override
	public boolean canStop() {
		return false;
	}
	
	@Override
	public void stop() {
		// do nothing
	}
	
	protected abstract FileFilter[] getFilters();
	
	protected FileFilter getPreferredFilter() {
		return null;
	}

	private String getDotExtension(FileFilter ff) {
		
		if (!(ff instanceof FileNameExtensionFilter))
			return null;
		
		String[] exts = ((FileNameExtensionFilter)ff).getExtensions();
		
		if (exts.length > 0)
			return exts[0];
		
		return null;
	}
	
	@Override
	public boolean setSource() {
		JFileChooser f = new JFileChooser(lastDir==null?defaultDir:lastDir);
		f.setMultiSelectionEnabled(true);

		if (getFilters() == null)
			return false;
		
		FileFilter preferredFilter = getPreferredFilter();
		
		for (FileFilter filter: getFilters()) {
			if (filter != preferredFilter)
				f.addChoosableFileFilter(filter);
		}
		
		if (preferredFilter != null)
			f.setFileFilter(preferredFilter);
		
		if ((isLoadingAction() ? f.showOpenDialog(null) : f
				.showSaveDialog(null)) != JFileChooser.APPROVE_OPTION)
			return false;

		ArrayList<File> selectedFiles = new ArrayList<File>();
		
		// auto-add extensions to files
		for (File sf: f.getSelectedFiles()) {
			String dotExtension = null;
			
			if (sf.getName().indexOf('.') < 0 && (dotExtension = getDotExtension(f.getFileFilter())) != null) {
				selectedFiles.add(new File(sf.getAbsoluteFile()+"."+dotExtension));
			}
			else
				selectedFiles.add(sf);
		}
		
		// get files we would overwrite
		if (!isLoadingAction()) {
			StringBuffer filesToOverwrite = new StringBuffer(); 
			
			for (File fi: selectedFiles) {
				if (fi.exists())
					filesToOverwrite.append("\n"+fi.getName());
			}
			
			// ask if we want to overwrite
			if (filesToOverwrite.length() != 0) {
				if (JOptionPane.showConfirmDialog(null, 
						"Overwrite the following file(s)?"+filesToOverwrite.toString(), "Overwrite files?", 
						JOptionPane.YES_NO_OPTION) != JOptionPane.YES_OPTION)
					return false;
			}
		}
		
		source.clear();
		lastDir = f.getSelectedFile().getParentFile();
		defaultDir = lastDir;
		this.selectedFilter = f.getFileFilter();
		
		source.addAll(selectedFiles);
		
		return true;
	}

	@Override
	public void setController(Cassette cassette, StopCallback s) {
		this.cassette = cassette;
	}
	
}
