package controller;

import model.Cassette;

public interface IOAction {

	// static
	/**
	 * @return The menu path of the action (excluding "File"), e.g. "Import", "Audio file" 
	 */
	public String[] getMenuPath();
	
	/**
	 * @return true if the action has to be stopped by the user (e.g. when recording, the user has to stop pressing)
	 */
	public boolean canStop();
	
	/**
	 * @param c The cassette (which will be used to add / read items from)
	 * @param s A callback for when the operation has finished (can be used e.g. to reset button states, or change a status label)
	 */
	public void setController(Cassette c, StopCallback s);
	
	/**
	 * @return true if this is a "loading" action, false if it is a "save" action. "Loading" actions are always enabled, "save" actions
	 * are only enabled when there is at least one tabl.
	 */
	public boolean isLoadingAction();
	
	// callbacks
	
	/**
	 * Callback for when the "source" button is pressed.
	 * 
	 * It will always be called immediately before start()
	 * 
	 * @return Whether a new source was set
	 */
	public boolean setSource();
	
	/**
	 * Callback for when the action has to be started.
	 */
	public void start();
	/**
	 * Callback for when the action has to be stopped. Undefined behavior if not used (canStop is false).
	 */
	public void stop();

}
