package controller.loaders;

import gui.CassetteLogger;

import java.io.File;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import controller.Demodulator;
import controller.FileAction;
import controller.WaveCleaner;

public class AudioFileLoader extends FileAction {

	private boolean filter;
	
	public AudioFileLoader(boolean useV2, boolean filter) {
		this.filter = filter;
	}
	
	public AudioFileLoader() {
		this(true, false);
	}
	
	@Override
	protected FileFilter[] getFilters() {
		return new FileFilter[] { 
				new FileNameExtensionFilter("Audio files (*.wav, *.flac)", "wav", "flac")
				};
	}

	@Override
	public boolean isLoadingAction() {
		return true;
	}

	@Override
	public String[] getMenuPath() {
		return new String[]{"Import", "Audio file"};
	}

	@Override
	public void start() {
		for (File f: source) {
			// load file
			Demodulator p = cassette.getOptions().getDemodulator();
			
			AudioInputStream i;
			AudioFormat format = null;
			try {
				i = AudioSystem.getAudioInputStream(f);
				if (filter)
					i = new WaveCleaner(i);
				
				format = i.getFormat();
				
				p.reset();
				p.parse(i);
				CassetteLogger.info("Loaded "+f.getName());
				
				i.close();
			} catch (Exception e) {
				if (e instanceof IllegalArgumentException) {
					JOptionPane.showMessageDialog(null, "File "+f.getName()+" is in an unsupported format.\nPlease convert it to 16-bit mono first.");
					
					CassetteLogger.warn("Could not load "+f.getName()+": "+e.getMessage());
					CassetteLogger.debug("Format: "+format.getSampleRate()+"Hz, "+format.getSampleSizeInBits()+
							" bits per sample, channels: "+format.getChannels()+", big endian: "+format.isBigEndian()+", encoding: "+format.getEncoding());
					
				}
				else {
					CassetteLogger.warn("Error while loading "+f.getName()+":\n"+e.getClass().getName()+": "+e.getMessage());
					
					e.printStackTrace();
				}
			}
		}
	}

}
