package controller.loaders;

import gui.CassetteLogger;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.TargetDataLine;
import javax.sound.sampled.Mixer.Info;
import javax.swing.JOptionPane;

import model.Cassette;
import controller.Demodulator;
import controller.IOAction;
import controller.StopCallback;

public class CaptureLoader implements IOAction {

	private Cassette c;
	private Demodulator p;
	private TargetDataLine mic;
	private AudioInputStream micStream;
	private Info mixerInfo;
	private boolean customSource;
	
	public CaptureLoader(boolean customSource) {
		this.customSource = customSource;
	}
	
	@Override
	public boolean isLoadingAction() {
		return true;
	}
	
	@Override
	public boolean canStop() {
		return true;
	}

	@Override
	public String[] getMenuPath() {
		return new String[]{"Import", "From audio capture"+(customSource?" (custom source)":"")};
	}

	@Override
	public void setController(Cassette c, StopCallback s) {
		this.c = c;
	}

	@Override
	public boolean setSource() {
		if (!customSource)
			return true;
		
		// TODO
		StringBuffer sb = new StringBuffer();
		sb.append("Please choose an appropriate mixer:\n");
		sb.append("0. (Default mixer)\n");
		int n = 1;
		
		Info[] mixers = AudioSystem.getMixerInfo();
		
		for (Info i: AudioSystem.getMixerInfo()) {
			sb.append(n+". "+i.getName()+" - "+i.getDescription()+"\n");
			n++;
		}
		
		String opt = JOptionPane.showInputDialog(sb.toString(), "0");
		
		if (opt != null) {
			try {
				int pos = Integer.parseInt(opt);
				
				if (pos > 0 && pos <= mixers.length)
					mixerInfo = mixers[pos-1];
				else if (pos == 0)
					mixerInfo = null;
				
				return true;
			} catch (NumberFormatException e) {
				// wrong input, ignore
			}
		}

		return false;
	}

	@Override
	public void start() {
		p = c.getOptions().getDemodulator();
		
		try {
			AudioFormat f = new AudioFormat(48000, 16, 1, true, true);
			
			if (mixerInfo != null)
				mic = AudioSystem.getTargetDataLine(f, mixerInfo);
			else
				mic = AudioSystem.getTargetDataLine(f);
			
			mic.open();
			mic.start();
			micStream = new AudioInputStream(mic);
			p.reset();
			p.parse(micStream);
		} catch (Exception e) {
			CassetteLogger.warn("Error while capturing "+":\n"+e.getClass().getName()+": "+e.getMessage());
		}
	}

	@Override
	public void stop() {
		p.stop();
		mic.close();
	}

}
