package controller.loaders;

import gui.EasyLayout;

import java.awt.MouseInfo;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedHashMap;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JTextField;

import model.Cassette;
import model.PArray;
import model.PBasicProgram;
import model.PData;
import model.PImage;
import model.PMachineCode;
import model.PRawData;
import model.PString;
import controller.IOAction;
import controller.StopCallback;

public class TemplateLoader implements IOAction {

	private Cassette c;
	private String name = "";
	private String type = "";
	private boolean accepted = false;
	private JTextField nameField = new JTextField(15);
	private LinkedHashMap<String, Class<? extends PData>> types = new LinkedHashMap<String, Class<? extends PData>>();
	private JComboBox typeField = new JComboBox();
	private JDialog typeDialog = new JDialog();
	
	public TemplateLoader() {
		// fill in the types
		types.put("BASIC Program", PBasicProgram.class);
		types.put("Array", PArray.class);
		types.put("Video Memory", PImage.class);
		types.put("Machine Code", PMachineCode.class);
		types.put("String", PString.class);
		types.put("Raw data", PRawData.class);
		
		for (String s: types.keySet()) {
			typeField.addItem(s);
		}
		
		// create type dialog
		init();
	}
	
	@Override
	public boolean isLoadingAction() {
		return true;
	}
	
	public void init() {
		typeDialog.setModal(true);
		typeDialog.setTitle("Choose file type");
		
		JPanel p = new JPanel(new EasyLayout());
		p.add(new JLabel("Type:"), EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		p.add(typeField,EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		
		p.add(new JLabel("Name:"), EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		p.add(nameField, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		
		//p.add(new JSeparator(), EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		
		JButton okButton = new JButton("OK");
		typeDialog.getRootPane().setDefaultButton(okButton);
		
		okButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				type = (String)(typeField.getSelectedItem());
				name = nameField.getText();
				
				accepted = true;
				typeDialog.setVisible(false);
			}
		});
	
		p.add(new JLabel(""), EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		p.add(okButton);
		
		typeDialog.setContentPane(p);
		typeDialog.pack();
	}
	
	@Override
	public boolean canStop() {
		return false;
	}

	@Override
	public String[] getMenuPath() {
		return new String[]{"New..."};
	}

	@Override
	public void setController(Cassette c, StopCallback s) {
		this.c = c;
	}

	@Override
	public boolean setSource() {
		accepted = false;
		
		nameField.setText(name);
		typeField.setSelectedItem(type);
		nameField.selectAll();
		typeField.requestFocusInWindow();
		Point pointer = MouseInfo.getPointerInfo().getLocation();
		typeDialog.setLocation(pointer.x, pointer.y); // TODO better positioning
		
		typeDialog.setVisible(true);
		return accepted;
	}

	@Override
	public void start() {
		PData b = null;
		try {
			b = types.get(type).newInstance();
		} catch (InstantiationException e) {
			// shouldn't happen
		} catch (IllegalAccessException e) {
			// shouldn't happen
		}
		
		b.setName(name);
		c.addBlock(b, true);
	}

	@Override
	public void stop() {
		throw new UnsupportedOperationException();
	}

}
