package controller.loaders;

import gui.CassetteLogger;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import model.PData;
import controller.FileAction;

public class ZipFileLoader extends FileAction {

	@Override
	protected FileFilter[] getFilters() {
		return new FileFilter[] { 
				new FileNameExtensionFilter("Zip files (*.zip)", "zip")
				};
	}

	@Override
	public boolean isLoadingAction() {
		return true;
	}
	
	@Override
	public String[] getMenuPath() {
		return new String[]{"Import", "Zip Archive"};
	}

	@Override
	public void start() {
		for (File f: source) {
			// open zip file
			try {
				ZipFile zf = new ZipFile(f);
				
				Enumeration<? extends ZipEntry> entries = zf.entries();
				
				// add all elements
				while (entries.hasMoreElements()) {
					ZipEntry e = entries.nextElement();
					InputStream zinput = zf.getInputStream(e);
					
					cassette.addBlock(PData.fromNative(e.getName(), zinput), true);
					CassetteLogger.debug("Loaded "+e.getName()+" from "+f.getName());
				}
				CassetteLogger.info("Loaded "+f.getName());
				
			} catch (IOException e) {
				CassetteLogger.warn("Error while loading "+f.getName()+":\n"+e.getClass().getName()+": "+e.getMessage());
				continue;
			}
		}
	}

}
