package controller.savers;

import gui.CassetteLogger;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;

import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.kc7bfi.jflac.sound.spi.FlacFileFormatType;

import controller.FileAction;
import controller.WaveEncoder;

public class AudioFileSaver extends FileAction {

	private boolean isSingle;

	public AudioFileSaver(boolean isSingle) {
		this.isSingle = isSingle;
	}
	
	@Override
	protected FileFilter[] getFilters() {
		return new FileFilter[] { 
				new FileNameExtensionFilter("Audio files (*.wav)", "wav")
				// TODO flac
				};
	}

	@Override
	public boolean isLoadingAction() {
		return false;
	}
	
	@Override
	public String[] getMenuPath() {
		return new String[]{"Export", "Audio file"+(isSingle?" (selected tab)":"")};
	}

	@Override
	public void start() {
		AudioFormat targetFormat = new AudioFormat(48000, 16, 1, true, false);
		
		for (File f: source) {
			WaveEncoder e = new WaveEncoder(cassette, false);
			AudioFileFormat.Type type;
			
			if (f.getName().toLowerCase().endsWith(".flac"))
				type = FlacFileFormatType.FLAC;
			else
				type = AudioFileFormat.Type.WAVE;
			
			ByteArrayOutputStream out = new ByteArrayOutputStream(16384);
						
			try {
				// get all bytes
				if (isSingle) {
					e.encodeSelectedTab(out);
					out.close();
				}
				else {
					e.encode(out);
					out.close();
				}
				
				// convert to audio
				AudioInputStream in = new AudioInputStream(
						new ByteArrayInputStream(out.toByteArray()),
						targetFormat, AudioSystem.NOT_SPECIFIED);
				
				// write
				AudioSystem.write(in, type, f);
				CassetteLogger.info("Saved "+f.getName());

			} catch (Exception ex) {
				CassetteLogger.warn("Could not save to "+f.getName()+":\n"+ex.getMessage());
				ex.printStackTrace();
			}
		}
	}

}
