package controller.savers;

import gui.CassetteLogger;
import gui.MainPanel;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.HashMap;

import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import model.PData;
import controller.FileAction;
import controller.savers.migration.CSVMigrator;
import controller.savers.migration.GIFMigrator;
import controller.savers.migration.Migrator;
import controller.savers.migration.PNGMigrator;

public class MigrationSaver extends FileAction {

	private HashMap<Migrator, FileNameExtensionFilter> registeredFilters = new HashMap<Migrator, FileNameExtensionFilter>();
	private MainPanel p; // EWWW!!!
	
	public MigrationSaver(MainPanel p) {
		registerFilter(new CSVMigrator(false));
		registerFilter(new CSVMigrator(true));
		registerFilter(new PNGMigrator());
		registerFilter(new GIFMigrator());
		this.p = p;
	}
	
	private void registerFilter(Migrator m) {
		registeredFilters.put(m, new FileNameExtensionFilter(m.getDescription(), m.getExtension()));
	}
	
	@Override
	protected FileFilter[] getFilters() {
		PData d = cassette.getSelected();

		ArrayList<FileFilter> filters = new ArrayList<FileFilter>();
		
		for (Migrator m: registeredFilters.keySet()) {
			if (m.canMigrate(d))
				filters.add(registeredFilters.get(m));
		}
		
		if (filters.size() == 0)
			return null;
		else
			return filters.toArray(new FileFilter[0]);
	}
	
	public boolean accept(PData d) {

		if (d == null)
			return false;
		
		for (Migrator m: registeredFilters.keySet()) {
			if (m.canMigrate(d))
				return true;
		}
		
		return false;
	}

	@Override
	public String[] getMenuPath() {
		return new String[]{"Migrate selected tab"};
	}

	@Override
	public boolean isLoadingAction() {
		return false;
	}

	@Override
	public void start() {
		for (File f: source) {
			if (!registeredFilters.containsValue(selectedFilter)) {
				CassetteLogger.warn("Could not migrate to unknown format!");
				return;
			}
			
			for (Migrator m: registeredFilters.keySet()) {
				if (registeredFilters.get(m) == selectedFilter) {
					try {
						FileOutputStream fo = new FileOutputStream(f);
						m.migrate(cassette.getSelected(), fo, p.getEditorTab(cassette.getSelectedIndex()));
						fo.close();
						
						CassetteLogger.info("Migrated to "+m.getDescription()+": "+f.getName());
					} catch (Exception e) {
						e.printStackTrace();
						CassetteLogger.warn("Could not migrate "+f.getName()+" ("+m.getDescription()+"):\n "+e.getClass().getName()+": "+e.getMessage());
					}
				}
			}
		}
	}

}
