package controller.savers;

import gui.CassetteLogger;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import model.PData;
import controller.FileAction;

public class NativeFileSaver extends FileAction {
	private boolean saveAs = false;
	
	public NativeFileSaver() {
		this(true);
	}
	
	public NativeFileSaver(boolean saveAs) {
		this.saveAs = saveAs;
	}
	
	@Override
	protected FileFilter[] getFilters() {
		return new FileFilter[] { 
				new FileNameExtensionFilter("Strings (*.str)", "str"),
				new FileNameExtensionFilter("Arrays (*.arr)", "arr"),
				new FileNameExtensionFilter("Video Memory (*.img)", "img"),
				new FileNameExtensionFilter("Machine Code (*.bin)", "bin"),
				new FileNameExtensionFilter("Binary Format data (*.dat)", "dat"),
				new FileNameExtensionFilter("Raw data (*.raw)", "raw"),
				new FileNameExtensionFilter("BASIC Files (*.bas, *.txt)", "bas", "txt"),
		};
	}
	
	@Override
	protected FileFilter getPreferredFilter() {
		String ext = cassette.getSelected().getDefaultExtension();
		
		for (FileFilter f: getFilters()) {
			if (f.accept(new File("file."+ext))) // XXX this is kind of a hack!
				return f;
		}
		return null;
	}

	@Override
	public boolean setSource() {
		if (!saveAs && cassette.getSelected().getDiskFiles() != null) {
			source = new ArrayList<File>();
			for (File f: cassette.getSelected().getDiskFiles())
				source.add(f);
		}
		else {
			if (super.setSource())
				cassette.getSelected().setDiskFiles(source.toArray(new File[0]));
			else
				return false;
		}
		
		return true;
	}
	
	@Override
	public boolean isLoadingAction() {
		return false;
	}
	
	@Override
	public String[] getMenuPath() {
		return new String[]{saveAs?"Save as...":"Save"};
	}
	
	@Override
	public void start() {

		PData b = cassette.getSelected();
		
		for (File f: source) {
			
			String extension = f.getName().replaceFirst("^.*\\.([^.]+)$", "$1").toLowerCase();
			
			if (!extension.equals(b.getDefaultExtension()) && 
					JOptionPane.showConfirmDialog(null, "Chosen extension for "+f.getName()+
							" may be inadequate.\nThe default extension for this data format is '"+
							b.getDefaultExtension()+"'\nContinue anyway?", "Inadequate extension",
							JOptionPane.YES_NO_OPTION) != JOptionPane.YES_OPTION)
				continue;
			
			
			try {
				FileOutputStream o = new FileOutputStream(f);
				b.toOutputStream(o);
				o.close();
				CassetteLogger.info("Saved "+f.getName());
				b.setModified(false);
			} catch (IOException e) {
				CassetteLogger.warn("Could not save "+b.getName()+" to "+f.getName()+":\n"+e.getMessage());
			}
		}
	}
}
