package controller.savers;

import gui.CassetteLogger;

import java.io.IOException;
import java.io.OutputStream;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.SourceDataLine;

import model.Cassette;
import controller.IOAction;
import controller.StopCallback;
import controller.WaveEncoder;

public class PlaybackSaver implements IOAction {

	private Cassette c;
	private StopCallback s;
	private boolean isSingle;
	private SourceDataLine line;
	private WaveEncoder enc;
	
	public PlaybackSaver(boolean isSingle) {
		this.isSingle = isSingle;
	}
	
	@Override
	public boolean isLoadingAction() {
		return false;
	}
	
	@Override
	public boolean canStop() {
		return true;
	}

	@Override
	public String[] getMenuPath() {
		return new String[]{"Export", "To audio playback"+(isSingle?" (selected tab)":"")};
	}
	
	@Override
	public void setController(Cassette c, StopCallback s) {
		this.c = c;
		this.s = s;
	}

	@Override
	public boolean setSource() {
		// do nothing (for now)
		return true;
	}

	@Override
	public void start() {

		try {
			AudioFormat targetFormat = new AudioFormat(48000, 16, 1, true, false);
			line = AudioSystem.getSourceDataLine(targetFormat);
			enc = new WaveEncoder(c, false);

			line.open();
			line.start();

			PassthroughOutputStream out = new PassthroughOutputStream(line);

			if (isSingle)
				enc.encodeSelectedTab(out);
			else
				enc.encode(out);

			//line.flush();
			s.stop();

			CassetteLogger.info("Playback complete");
		} catch (Exception e) {
			CassetteLogger.warn("Could not play back:\n"+e.getClass().getName()+": "+e.getMessage());
			s.stop();
		}

	}

	public class PassthroughOutputStream extends OutputStream {
		private SourceDataLine line;
		
		public PassthroughOutputStream(SourceDataLine line) {
			this.line = line;
		}
		
		@Override
		public void write(int b) throws IOException {
			byte[] w = new byte[1];
			w[0] = (byte)b;
			line.write(w, 0, 1);
		}
		
		@Override
		public void write(byte[] b) throws IOException {
			line.write(b, 0, b.length);
		}
		
		@Override
		public void write(byte[] b, int off, int len) throws IOException {
			line.write(b, off, len);
		}
	}
	
	boolean stopped = false;
	
	@Override
	public synchronized void stop() {
		if (stopped)
			return;
		
		stopped = true;
		enc.stop();
		
		// assert playback slower than conversion - we might leak some bytes otherwise
		line.drain();
		line.stop();
		line.close();
	}

}
