package controller.savers;

import gui.CassetteLogger;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashSet;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import model.PData;
import controller.FileAction;

public class ZipFileSaver extends FileAction {

	@Override
	protected FileFilter[] getFilters() {
		return new FileFilter[] { 
				new FileNameExtensionFilter("Zip files (*.zip)", "zip")
				};
	}

	@Override
	public boolean isLoadingAction() {
		return false;
	}
	
	@Override
	public String[] getMenuPath() {
		return new String[]{"Export", "Zip Archive"};
	}

	@Override
	public void start() {
		String currentData = "ZIP header";
		HashSet<String> usedNames = new HashSet<String>();
		
		for (File f: source) {
			// open zip file - write to every selected zip file
			try {
				ZipOutputStream zo = new ZipOutputStream(new FileOutputStream(f));
		
				for (PData d: cassette) {
					currentData = d.getName();
					
					// save data blocks called the same in files mid-fixed with a sequence number (block.number.extension)
					String currentName = d.getName()+"."+d.getDefaultExtension();
					int i = 0;
					
					while (usedNames.contains(currentName)) {
						i++;
						currentName = d.getName()+"."+i+"."+d.getDefaultExtension();
					}
					
					usedNames.add(currentName);
					
					zo.putNextEntry(new ZipEntry(currentName));
					d.toOutputStream(zo);
					zo.closeEntry();
					CassetteLogger.debug("Saved "+currentName+" to "+f.getName());
				}
				
				zo.close();
				CassetteLogger.info("Saved "+f.getName());
			} catch (IOException e) {
				CassetteLogger.warn("Could not save "+currentData+" to "+f.getName()+":\n"+e.getMessage());
			}
		}
	}

}
