package controller.savers.migration;

import gui.PDataEditor;
import gui.editor.ArrayTable;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.text.DecimalFormat;
import java.util.ArrayList;

import model.PArray;
import model.PData;

public class CSVMigrator implements Migrator {

	public boolean matrix;
	
	public CSVMigrator(boolean matrix) {
		this.matrix = matrix;
	}
	
	@Override
	public String getDescription() {
		return "CSV File"+(matrix?" (as 2D-matrix)":"");
	}

	@Override
	public String getExtension() {
		return "csv";
	}

	@Override
	public boolean canMigrate(Object obj) {
		return (obj instanceof PArray && (!matrix || ((PArray)obj).getExtraDimensions().size() == 1));
	}

	@Override
	public void migrate(PData data, OutputStream s, PDataEditor editor)
			throws IOException {
		if (!(data instanceof PArray))
			throw new IllegalArgumentException();
		
		PArray arr = (PArray)data;
		PrintWriter p = new PrintWriter(s);
		
		DecimalFormat df = new DecimalFormat("#.#######");
		if (matrix) {
			int cols = arr.getExtraDimensions().get(0);
			int i = 0;
			// just write the numbers / whatever, no headers
			String[] row = new String[cols];
			
			for (Long l: arr.getArray()) {
				row[i] = arr.isNumber()?df.format(PArray.toFloat(l)):("0x"+Long.toHexString(l));
				
				i++;
				if (i == cols) {
					p.println(join(row, ","));
					i = 0;
				}
			}
		}
		else {
			// write headers
			ArrayList<Integer> extraDimensions = arr.getExtraDimensions();
			
			String[] row = new String[extraDimensions.size()+2];
			for (int i = 0; i < extraDimensions.size()+1; i++) {
				row[i] = "\"Dimension "+(i+1)+"\"";
			}
			row[row.length - 1] = "\"Value\"";
			
			p.println(join(row, ","));
			
			int n = 0;
			for (Long l: arr.getArray()) {
				String[] dims = ArrayTable.getPosition(n, extraDimensions).split(" ");
				System.arraycopy(dims, 0, row, 0, dims.length);
				row[dims.length] = arr.isNumber()?df.format(PArray.toFloat(l)):("0x"+Long.toHexString(l));
				
				p.println(join(row, ","));
				n++;
			}
		}
		
		p.flush();
		
	}
 
	public static String join(String[] s, String sep) {
		boolean first = true;
		StringBuffer sb = new StringBuffer();
		
		for (String cs: s) {
			if (!first)
				sb.append(sep);
			first = false;
			
			sb.append(cs);
		}
		
		return sb.toString();
	}
	
}
