package controller.savers.migration;

import gui.PDataEditor;
import gui.widgets.ImageEditor;

import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Iterator;

import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageTypeSpecifier;
import javax.imageio.ImageWriter;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.metadata.IIOMetadataNode;
import javax.imageio.stream.ImageOutputStream;

import model.PData;
import model.PImage;

import org.w3c.dom.Node;

public class GIFMigrator implements Migrator {

	@Override
	public String getDescription() {
		return "Animated GIF";
	}

	@Override
	public String getExtension() {
		return "gif";
	}

	@Override
	public boolean canMigrate(Object obj) {
		return obj instanceof PImage;
	}

	@Override
	public void migrate(PData data, OutputStream s, PDataEditor editor) throws IOException {
		if (!(data instanceof PImage) || editor == null || !(editor instanceof ImageEditor))
			throw new IllegalArgumentException("Internal error, try again"
					/*"PImage: "+(data instanceof PImage)+", editor non-null: "+(editor != null)*/);
		
		//PImage im = (PImage)data;
		ImageEditor ie = (ImageEditor)editor; 
		
		Iterator<ImageWriter> gifWriters = ImageIO.getImageWritersByFormatName("gif");
		
		if (!gifWriters.hasNext()) // uh-oh!
			throw new UnsupportedOperationException();
		
		ImageOutputStream ios = ImageIO.createImageOutputStream(s);
		ImageWriter gifWriter = gifWriters.next();
		
		Dimension d = ie.getTableDimensions();
		BufferedImage bi = new BufferedImage(d.width, d.height, BufferedImage.TYPE_BYTE_INDEXED);
		
		gifWriter.setOutput(ios);
		gifWriter.prepareWriteSequence(null);
		
		IIOMetadata m = gifWriter.getDefaultImageMetadata(new ImageTypeSpecifier(bi), null);
		Node node = m.getAsTree(m.getNativeMetadataFormatName());
		IIOMetadataNode gcNode = new IIOMetadataNode("GraphicControlExtension");
		gcNode.setAttribute("disposalMethod", "none");
		gcNode.setAttribute("userInputFlag", "FALSE");
		gcNode.setAttribute("transparentColorFlag", "FALSE");
		gcNode.setAttribute("delayTime", "100");
		gcNode.setAttribute("transparentColorIndex", "0");
		node.appendChild(gcNode);
		
	    IIOMetadataNode aesNode =  new IIOMetadataNode("ApplicationExtensions");
	    IIOMetadataNode aeNode = new IIOMetadataNode("ApplicationExtension");

	    aeNode.setAttribute("applicationID", "NETSCAPE");
	    aeNode.setAttribute("authenticationCode", "2.0");

	    aeNode.setUserObject(new byte[]{ 0x1, 0x00, (byte)0x00}); // loop
	    aesNode.appendChild(aeNode);
	    node.appendChild(aesNode);
	    m.setFromTree(m.getNativeMetadataFormatName(), node);

		ie.paintTable(bi.getGraphics(), false);
		gifWriter.writeToSequence(new IIOImage(bi, null, m), null);
		
		ie.paintTable(bi.getGraphics(), true);
		gifWriter.writeToSequence(new IIOImage(bi, null, m), null);
		
		gifWriter.endWriteSequence();
		
		ios.flush();
	}

}
