package gui;

import java.awt.Color;
import java.util.LinkedList;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

public class CassetteLogger {

	private static LinkedList<StyledDocument> documents = new LinkedList<StyledDocument>(); 
	private static LinkedList<LoggerMeter> meters = new LinkedList<LoggerMeter>(); 
	
	
	private static SimpleAttributeSet info = new SimpleAttributeSet();
	private static SimpleAttributeSet warn = new SimpleAttributeSet();
	private static SimpleAttributeSet debug = new SimpleAttributeSet();
	
	static {
		info.addAttribute(StyleConstants.Bold, true);
		warn.addAttribute(StyleConstants.Bold, true);
		warn.addAttribute(StyleConstants.Foreground, Color.RED);
		// debug is plain
		//debug.addAttribute(StyleConstants.Italic, true);
	}
	
	private synchronized static void output(String msg, AttributeSet a) {
		//System.err.println(msg);
		for (StyledDocument d: documents) {
			try {
				d.insertString(d.getLength(), msg+"\n", a);
			} catch (BadLocationException e) {
				// shouldn't happen
			}
		}
	}
	
	public synchronized static void info(String msg) {
		output(msg, info);
	}
	
	public synchronized static void warn(String msg) {
		output(msg, warn);
	}
	
	public synchronized static void debug(String msg) {
		output(msg, debug);
	}
	
	public synchronized static void clear() {
		for (StyledDocument d: documents) {
			try {
				d.remove(0, d.getLength());
			} catch (BadLocationException e) {
				// shouldn't happen
			}
		}
	}
	
	public static void addDocument(StyledDocument doc) {
		documents.add(doc);
	}
	
	public static void removeDocument(StyledDocument doc) {
		documents.remove(doc);
	}
	
	
	// meters
	public synchronized static void setMeter(int percent) {
		for (LoggerMeter m: meters) {
			m.setMeter(percent);
		}
	}
	
	public synchronized static void setAuxMeter(int percent) {
		for (LoggerMeter m: meters) {
			m.setAuxMeter(percent);
		}
	}
	
	public static void addMeter(LoggerMeter m) {
		meters.add(m);
	}

	public static void removeMeter(LoggerMeter m) {
		meters.remove(m);
	}
}
