package gui;

import gui.widgets.WidgetSyncListener;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JToggleButton;

import controller.IOAction;
import controller.StopCallback;

@SuppressWarnings("serial")
public class IOPanel extends JPanel implements ActionListener, StopCallback {

	private String description;
	private String action;
	
	private JComboBox optionBox;
	private JButton sourceButton;
	private JLabel sourceLabel;
	private JToggleButton startButton;
	
	private WidgetSyncListener listener;
	
	private ArrayList<IOAction> actions = new ArrayList<IOAction>();
	private IOAction selectedAction;
	
	private static final String OPTION_CHANGED = "option";
	private static final String BUTTON_TOGGLED = "button";
	private static final String SOURCE_CHANGE = "source";
	
	
	public IOPanel(String description, String action, WidgetSyncListener listener) {
		this.description = description;
		this.action = action;
		this.listener = listener;
		
		init();
	}
	
	private void init() {
		
		//SpringLayout layout = new SpringLayout();
		setBorder(BorderFactory.createCompoundBorder(
				BorderFactory.createEmptyBorder(5, 5, 5, 5),
				BorderFactory.createTitledBorder(action)));

		BorderLayout layout = new BorderLayout();
		setLayout(layout);
		
		JPanel centerPanel = new JPanel(new GridLayout(0,1));
		add(centerPanel, BorderLayout.CENTER);
		
		JLabel descriptionLabel = new JLabel(description);
		centerPanel.add(descriptionLabel);

		optionBox = new JComboBox();
		centerPanel.add(optionBox);
		optionBox.setActionCommand(OPTION_CHANGED);
		optionBox.addActionListener(this);
		
		sourceLabel = new JLabel("Source");
		centerPanel.add(sourceLabel);
		
		JPanel bottomPanel = new JPanel(new GridLayout(1, 0, 5, 5));
		add(bottomPanel, BorderLayout.SOUTH);
		
		sourceButton = new JButton("Source");
		sourceButton.setActionCommand(SOURCE_CHANGE);
		sourceButton.addActionListener(this);
		
		startButton = new JToggleButton(action);
		startButton.setActionCommand(BUTTON_TOGGLED);
		startButton.addActionListener(this);
		
		bottomPanel.add(sourceButton);
		bottomPanel.add(startButton);
		
		/*
		layout.putConstraint(SpringLayout.NORTH, this, 5, SpringLayout.NORTH, descriptionLabel);
		layout.putConstraint(SpringLayout.EAST, this, 5, SpringLayout.EAST, descriptionLabel);
		
		layout.putConstraint(SpringLayout.NORTH, optionBox, 5, SpringLayout.SOUTH, descriptionLabel);*/
	}
	
	private void updateSelectedAction() {
		if (optionBox.getSelectedIndex() < 0 || optionBox.getSelectedIndex() >= actions.size())
			return;
		
		// option has changed, update source, source button
		selectedAction = actions.get(optionBox.getSelectedIndex());
		
		//sourceLabel.setText(selectedAction.getSource());
		//sourceButton.setText(selectedAction.getSourceLabel());
		//sourceButton.setEnabled(selectedAction.getSourceLabel() != null);
		//startButton.setEnabled(selectedAction.getSource().length() > 0);
	}
	
	public void addAction(IOAction action) {
		// add to option box
		optionBox.addItem(action.getMenuPath());
		actions.add(action);
		
		updateSelectedAction();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (OPTION_CHANGED.equals(e.getActionCommand())) {
			updateSelectedAction();
		}
		else if (BUTTON_TOGGLED.equals(e.getActionCommand()) && selectedAction != null) {
			if (startButton.isSelected()) {
				// sync editors
				listener.sync();
				
				// clear log
				CassetteLogger.clear();
				
				// starting
				selectedAction.start();
				
				if (!selectedAction.canStop())
					startButton.setSelected(false);
			}
			else {
				selectedAction.stop();
			}
		}
		else if (SOURCE_CHANGE.equals(e.getActionCommand()) && selectedAction != null) {
			boolean newSource = selectedAction.setSource();
			if (newSource) {
				// update label and possibly enable button
				updateSelectedAction();
				
				// stop previous action
				if (startButton.isSelected())
					stop();
				
				// trigger a click
				startButton.doClick();
			}
		}
	}
	
	/**
	 * Stop playback automatically
	 */
	@Override
	public void stop() {
		startButton.doClick();
	}
	
}
