package gui;

import gui.widgets.ArrayEditor;
import gui.widgets.BasicEditor;
import gui.widgets.HexEditor;
import gui.widgets.ImageEditor;
import gui.widgets.StringEditor;
import gui.widgets.WidgetSyncListener;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTabbedPane;
import javax.swing.JTextPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import model.Cassette;
import model.CassetteListener;
import model.PArray;
import model.PBasicProgram;
import model.PData;
import model.PImage;
import model.PMachineCode;
import model.PString;
import controller.IOAction;
import controller.loaders.AudioFileLoader;
import controller.loaders.CaptureLoader;
import controller.loaders.NativeFileLoader;
import controller.loaders.TemplateLoader;
import controller.loaders.ZipFileLoader;
import controller.savers.AudioFileSaver;
import controller.savers.HexSaver;
import controller.savers.NativeFileSaver;
import controller.savers.PlaybackSaver;
import controller.savers.ZipFileSaver;

@SuppressWarnings("serial")
public class MainPanelOld extends JPanel implements CassetteListener, WidgetSyncListener, ChangeListener {

	private Cassette cassette;
	private IOPanel importPanel;
	private IOPanel exportPanel;
	private JTabbedPane tabs;
	private JProgressBar meter;
	private JProgressBar auxMeter;

	
	public MainPanelOld() {
		cassette = new Cassette();
		cassette.addListener(this);

		init();
	}

	private void init() {

		setLayout(new BorderLayout());

		JPanel topPanel = new JPanel(new BorderLayout());
		tabs = new JTabbedPane();
		
		tabs.addChangeListener(this);
		
		JPanel bottomPanel = new JPanel(new BorderLayout());

		add(topPanel, BorderLayout.NORTH);
		add(tabs, BorderLayout.CENTER);
		add(bottomPanel, BorderLayout.SOUTH);

		JButton exitButton = new JButton("Exit application");
		exitButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (JOptionPane.showConfirmDialog(MainPanelOld.this,
						"Are you sure you want to exit?") == JOptionPane.YES_OPTION)
					System.exit(0);
			}
		});

		bottomPanel.add(exitButton, BorderLayout.EAST);

		// add logging area
		JPanel logPanel = new JPanel(new BorderLayout());
		
		// textual logging pane
		JTextPane logPane = new JTextPane();
		logPane.setEditable(false);
		
		CassetteLogger.addDocument(logPane.getStyledDocument());
		JScrollPane logScroll = new JScrollPane(logPane);
		// TODO: predefined size, ugly...
		logScroll.setPreferredSize(new Dimension(360, 120));
		
		logPanel.add(logScroll, BorderLayout.CENTER);
		
		// meters - in their own panel
		JPanel meterPanel = new JPanel(new GridLayout(0,1));
		logPanel.add(meterPanel, BorderLayout.SOUTH);
		
		meter = new JProgressBar();
		meter.setBorderPainted(false);

		auxMeter = new JProgressBar();
		auxMeter.setForeground(Color.RED);
		auxMeter.setBorderPainted(false);

		CassetteLogger.addMeter(new LoggerMeter() {
			@Override
			public void setMeter(int percent) {
				meter.setValue(percent);
			}
			
			@Override
			public void setAuxMeter(int percent) {
				auxMeter.setValue(percent);

			}
		});
		
		meterPanel.add(meter);
		meterPanel.add(auxMeter);
		
		// add whole log panel
		topPanel.add(logPanel, BorderLayout.CENTER);
		
		// add import and export panels
		importPanel = new IOPanel("Import data from:", "Import", this);
		exportPanel = new IOPanel("Export data to:", "Export", this);

		topPanel.add(importPanel, BorderLayout.WEST);
		topPanel.add(exportPanel, BorderLayout.EAST);

		// add actions (load from file, ...) here
		
		addAction(importPanel, new TemplateLoader());
		addAction(importPanel, new ZipFileLoader());
		addAction(importPanel, new NativeFileLoader());
		addAction(importPanel, new CaptureLoader(true));
		//addAction(importPanel, new AudioFileLoader());
		//addAction(importPanel, new AudioFileLoader(true, true));
		addAction(importPanel, new AudioFileLoader(true, false));
		//addAction(importPanel, new AudioFileLoader(false, true));
		//addAction(importPanel, new AudioFileLoader(false, false));
		
		addAction(exportPanel, new ZipFileSaver());
		addAction(exportPanel, new NativeFileSaver());
		addAction(exportPanel, new PlaybackSaver(true));
		addAction(exportPanel, new PlaybackSaver(false));
		addAction(exportPanel, new AudioFileSaver(true));
		addAction(exportPanel, new AudioFileSaver(false));
		addAction(exportPanel, new HexSaver());
		

		// add widget factories here
		
		componentMap.put(PArray.class, ArrayEditor.class);
		componentMap.put(PBasicProgram.class, BasicEditor.class);
		componentMap.put(PImage.class, ImageEditor.class);
		componentMap.put(PString.class, StringEditor.class);
		componentMap.put(PMachineCode.class, HexEditor.class);
		// hex editor uses the default
	}

	public void setMenu(JMenuBar b) {
		add(b, BorderLayout.SOUTH);
	}
	
	private void addAction(IOPanel panel, IOAction action) {
		action.setController(cassette, panel);
		panel.addAction(action);
	}

	@Override
	public void blockAdded(PData data) {
		// block added by ioaction, add tab		
		Component c = getWidget(data.getClass());
		
		Component addMe = c;
		
		// only add scrollbars if not PDataEditor or requested
		if (!(c instanceof PDataEditor) || ((PDataEditor)c).useScrollbars())
			addMe = new JScrollPane(c);
		
		tabs.addTab(null, addMe);
		tabs.setTabComponentAt(tabs.getTabCount()-1, new TabClosingPanel(addMe, data.getName(), data.getAddress()));
		
		// add data to c
		if (c instanceof PDataEditor) {
			((PDataEditor)c).setData(data);
		}
	}
	
	public void removeTab(Component c) {
		int idx = tabs.indexOfComponent(c);
		cassette.remove(idx);
		tabs.remove(idx);
	}

	public void moveTab(Component c) { // move a tab to the right
		int idx = tabs.indexOfComponent(c);
		int newIdx = (idx+1)%cassette.size();
		
		Component tabComponent = tabs.getTabComponentAt(idx);
		
		// move data
		PData data = cassette.remove(idx);
		cassette.add(newIdx, data);
		
		// move editor
		tabs.remove(c);
		tabs.insertTab(null, null, c, null, newIdx);
		tabs.setTabComponentAt(newIdx, tabComponent);
	}
	
	public void renameTab(Component c, String newName) {
		int idx = tabs.indexOfComponent(c);
		cassette.get(idx).setName(newName);
	}
	
	public int setTabAddress(Component c, Integer address) {
		int idx = tabs.indexOfComponent(c);
		cassette.get(idx).setAddress(address);
		return cassette.get(idx).getAddress();
	}
	
	@Override
	public void stateChanged(ChangeEvent e) {
		if (tabs.getSelectedIndex() >= 0)
			cassette.setSelected(tabs.getSelectedIndex());
	}
	
	private HashMap<Class<? extends PData>, Class<? extends Component>> componentMap = 
		new HashMap<Class<? extends PData>, Class<? extends Component>>();
	
	private Component getWidget(Class<? extends PData> c) {
		Component result = null;
		
		if (componentMap.containsKey(c)) {
			try {
				result = componentMap.get(c).newInstance();
			} catch (InstantiationException e) {
				// do nothing
			} catch (IllegalAccessException e) {
				// do nothing
			}
		}
		
		if (result == null)
			result = new HexEditor();
		
		return result;
	}
	
	@Override
	public void sync() {
		// sync all tabs
		
		for (Component c: tabs.getComponents()) {
			if (c instanceof JScrollPane) {
				c = ((JScrollPane)c).getViewport().getView();
			}
			
			if (c instanceof PDataEditor)
				((PDataEditor)c).syncData();
		}
		
	}

	private class TabClosingPanel extends JPanel implements ActionListener {
		private Component contents;
		private String title;
		private String address;
		private JButton addressButton;
		private JLabel l;
		
		public TabClosingPanel(Component contents, String title, int address) {
			super(new FlowLayout(FlowLayout.LEFT, 0, 0));
			setOpaque(false);
		
			this.contents = contents;
			this.title = title;
			this.address = Integer.toHexString(address);
			while (this.address.length() < 4)
				this.address = "0"+this.address;
			
			init();
		}
		
		private void init() {
			l = new JLabel(title+"   ");

			add(l);
			add(new JSeparator());
			
			addressButton = new JButton(address);
			addressButton.setFont(
					addressButton.getFont().deriveFont(
							addressButton.getFont().getSize2D()*0.8f));
			addressButton.setToolTipText("Set start address");
			addressButton.setBorderPainted(false);
			addressButton.setContentAreaFilled(false);
			addressButton.setActionCommand("address");
			addressButton.addActionListener(this);
			add(addressButton);
			
			JButton rename = new JButton("+");
			rename.setToolTipText("Rename tab");
			rename.setBorderPainted(false);
			rename.setContentAreaFilled(false);
			rename.setActionCommand("rename");
			rename.addActionListener(this);
			add(rename);
			
			JButton moveRight = new JButton("→");
			moveRight.setToolTipText("Move tab to the right");
			moveRight.setBorderPainted(false);
			moveRight.setContentAreaFilled(false);
			moveRight.setActionCommand("move");
			moveRight.addActionListener(this);
			add(moveRight);
			
			JButton b = new JButton("✖");
			b.setToolTipText("Close tab");
			b.setBorderPainted(false);
			b.setContentAreaFilled(false);
			b.setActionCommand("close");
			b.addActionListener(this);
			add(b);
		}
		
		@Override
		public void actionPerformed(ActionEvent e) {
			if ("move".equals(e.getActionCommand())) {
				MainPanelOld.this.moveTab(contents);
			}
			else if ("rename".equals(e.getActionCommand())) {
				String newName = JOptionPane.showInputDialog("Enter new name for "+title, title);
				if (newName != null) {
					title = newName;
					MainPanelOld.this.renameTab(contents, title);
					l.setText(title+"   ");
				}
			}
			else if ("address".equals(e.getActionCommand())) {
				String newAddress = JOptionPane.showInputDialog("Enter new start address for "+title, address);
				if (newAddress != null && newAddress.matches("[0-9a-fA-F]{1,4}")) {
					newAddress = Integer.toHexString(
							MainPanelOld.this.setTabAddress(contents, Integer.valueOf(newAddress, 16)));

					while (newAddress.length() != 4)
						newAddress = "0"+newAddress;
					
					address = newAddress;
					addressButton.setText(address);
				}
			} else { // close
				if (JOptionPane.showConfirmDialog(this, "Close '"+title+"'?") == JOptionPane.YES_OPTION)
					MainPanelOld.this.removeTab(contents);	
			}
			
		}
	}
	
}
