package gui;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSlider;
import javax.swing.JTextPane;

import controller.Demodulator;
import controller.OptionStore;

public class OptionPanel extends JPanel implements LoggerMeter, ActionListener, OptionStore, ItemListener {
	private static final long serialVersionUID = -6620711071356133289L;

	private JPanel actionPanel;
	
	private JCheckBox ignoreUnrecognized;
	private JCheckBox rawOutput;
	
	private JProgressBar meter;
	private JProgressBar auxMeter;
	
	private JSlider cutoff;
	private JSlider preamp;
	
	private JComboBox parser;
	
	private JCheckBox autoClear;
	private JButton clearButton;
	
	private Demodulator[] demodulators;
	
	private JLabel lastAction;
	private JButton startLastAction;
	
	private JButton closeTabs;
	
	public OptionPanel() {
		
		actionPanel = new JPanel(new EasyLayout());
		
		lastAction = new JLabel("");
		startLastAction = new JButton("Execute");
		startLastAction.setEnabled(false);
		
		closeTabs = new JButton("Close all tabs");
		
		// logging meter panel
		JPanel meterPanel = new JPanel(new GridLayout(0,1));
		meter = new JProgressBar();
		meter.setBorderPainted(false);
		
		auxMeter = new JProgressBar();
		auxMeter.setBorderPainted(false);
		
		meterPanel.add(meter);
		meterPanel.add(auxMeter);

		// add meters to the logging subsystem
		CassetteLogger.addMeter(this);
		
		// options
		ignoreUnrecognized = new JCheckBox("Ignore unrecongnized blocks");
		rawOutput = new JCheckBox("Only import raw data");
		parser = new JComboBox();
		
		parser.setPreferredSize(new Dimension(192, parser.getPreferredSize().height));
		
		parser.addItemListener(this);
		
		cutoff = new JSlider(0, 32768, 8192);
		preamp = new JSlider(0, 1000, 100); // pre-amplification filter (up to 1000% - default is 100%, meaning 16384 is max)
		
		// textual logging pane
		JTextPane logPane = new JTextPane();
		logPane.setEditable(false);
		
		// add to the logging subsystem
		CassetteLogger.addDocument(logPane.getStyledDocument());
		
		autoClear = new JCheckBox("Clear log automatically");
		autoClear.setSelected(true);
		
		clearButton = new JButton("Clear log");
		clearButton.addActionListener(this);
		
		JScrollPane logScroll = new JScrollPane(logPane);
		JPanel logPanel = new JPanel(new EasyLayout());
		logPanel.add(logScroll, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL+EasyLayout.VERTICAL_FILL);
		logPanel.add(autoClear, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		logPanel.add(clearButton);
		
		//logPanel.setBorder(BorderFactory.createTitledBorder("Log"));
		
		setLayout(new EasyLayout());
		
		add(lastAction, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		add(startLastAction);
		add(closeTabs);
		
		
		add(new JSeparator(), EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		
		add(actionPanel, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		
		add(meterPanel, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		
		add(ignoreUnrecognized, EasyLayout.NEW_ROW);
		add(rawOutput, EasyLayout.NEW_ROW);
		
		add(new JLabel("Demodulator:"), EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		add(parser);
		
		add(new JLabel("Signal cutoff:"), EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		add(cutoff);
		
		add(new JLabel("Pre-amplification:"), EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		add(preamp);
	
		add(logPanel, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL+EasyLayout.VERTICAL_FILL+EasyLayout.IGNORE_HORIZONTAL_WIDTH);
		
	}
	
	@Override
	public void setAuxMeter(int percent) {
		auxMeter.setValue(percent);
	}
	
	@Override
	public void setMeter(int percent) {
		meter.setValue(percent);
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		if (clearButton.equals(e.getSource())) {
			CassetteLogger.clear();
		}
	}
	
	public void setDemodulators(Demodulator[] demodulators) {
		this.demodulators = demodulators;
		
		while (parser.getItemCount() > 0)
			parser.removeItemAt(0);
		
		for (Demodulator d: demodulators)
			parser.addItem(d.getName());
		
		if (parser.getSelectedIndex() == -1)
			parser.setSelectedIndex(0);
		
		cutoff.setValue(demodulators[parser.getSelectedIndex()].getDefaultCutoff());
	}

	
	@Override
	public void itemStateChanged(ItemEvent e) {
		cutoff.setValue(demodulators[parser.getSelectedIndex()].getDefaultCutoff());
	}
	
	// option store data
	
	@Override
	public boolean autoClearLog() {
		return autoClear.isSelected();
	}

	@Override
	public Demodulator getDemodulator() {
		return demodulators[parser.getSelectedIndex()];
	}

	@Override
	public int getPreAmplification() {
		return preamp.getValue();
	}

	@Override
	public int getSignalCutoff() {
		return cutoff.getValue();
	}

	@Override
	public boolean ignoreUnrecognizedBlocks() {
		return ignoreUnrecognized.isSelected();
	}

	@Override
	public boolean onlyImportRaw() {
		return rawOutput.isSelected();
	}

	public JPanel getActionPanel() {
		return actionPanel;
	}
	
	public JButton getStartLastAction() {
		return startLastAction;
	}
	
	public void setLastAction(String action) {
		lastAction.setText(action);
	}
	
	public JButton getCloseTabs() {
		return closeTabs;
	}
}
