package gui.editor;
import java.awt.Color;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;


public class BasicDocument extends SyntaxDocument {
	private static final long serialVersionUID = -2211198181286020507L;
	
	SimpleAttributeSet line = new SimpleAttributeSet();
	SimpleAttributeSet keyword = new SimpleAttributeSet();
	SimpleAttributeSet string = new SimpleAttributeSet();
	SimpleAttributeSet number = new SimpleAttributeSet();
	SimpleAttributeSet comment = new SimpleAttributeSet();
	SimpleAttributeSet error = new SimpleAttributeSet();
	
    static String[] bascom =
    {
            "END","FOR","NEXT","DATA","INPUT","DIM","READ","LET",
            "GOTO","RUN","IF","RESTORE","GOSUB","RETURN","REM","STOP",
            "ON","LPRINT","DEF","POKE","PRINT","CONT","LIST","LLIST",
            "CLEAR","CLOAD","CSAVE","TX","GR","SCREEN","LINE","DISPLAY",
            "STORE","INIT","CURSORX","CURSORY","SCROLL","PAGE","BRIGHT","SOUND",
            "DELIM","SETET","SETEG","ET","EG","NEW","TAB(","TO",
            "FN","SPC(","THEN","NOT","STEP","+","-","*",
            "/","^","AND","OR",">","=","<","SGN",
            "INT","ABS","USR","FRE","LPOS","POS","SQR","RND",
            "LOG","EXP","COS","SIN","TAN","ATN","PEEK","LEN",
            "STR$","VAL","ASC","STICKX","STICKY","ACTION","KEY","CHR$",
            "LEFT$","RIGHT$","MID$"
    };
    
    private static Pattern keywordRegex;
    private static Pattern numberRegex;
    private static Pattern stringRegex;
    private static Pattern remRegex;
    private static Pattern openStringRegex;
    
    static {
    	StringBuffer sb = new StringBuffer();

    	for (String com: bascom) {
    		sb.append(Pattern.quote(com));
    		sb.append("|");
    	}
    	
    	sb.deleteCharAt(sb.length()-1);
    	
    	// hope this gets optimized
    	keywordRegex = Pattern.compile("\\b("+sb.toString()+")\\b", Pattern.CASE_INSENSITIVE);	
    	
    	numberRegex = Pattern.compile("[0-9]+");
    	stringRegex = Pattern.compile("\\\"[^\"]*\\\"");
    	remRegex = Pattern.compile("(^[0-9]+ +|:)REM (.*)", Pattern.CASE_INSENSITIVE);
    	openStringRegex = Pattern.compile("^([^\"]*\\\"[^\"]*\\\")*[^\"]*(\\\"[^\"]*)\n?$");
    	
    }
    
	public BasicDocument() {
		line.addAttribute(StyleConstants.Italic, true);
		
		keyword.addAttribute(StyleConstants.Bold, true);
		
		string.addAttribute(StyleConstants.Foreground, Color.RED);
		
		number.addAttribute(StyleConstants.Foreground, Color.BLUE);
		
		comment.addAttribute(StyleConstants.Foreground, Color.GRAY);
		
		//error.addAttribute(StyleConstants.Background, Color.RED);
		error.addAttribute(StyleConstants.Foreground, Color.DARK_GRAY);

	}
	
	@Override
	//Matcher m = Pattern.compile("^([0-9]+)( *)([a-zA-Z?]+)( *)(.*)$").matcher(line);
	
	protected List<TextAttribute> highlight(String line) {		
		LinkedList<TextAttribute> res = new LinkedList<TextAttribute>();
		
		// take care of line numbers + numbers
		Matcher numberMatcher = numberRegex.matcher(line);
		
		if (!numberMatcher.lookingAt()) {
			// no line number at the beginning, error
			res.add(new TextAttribute(0, line.length(), this.error));
			return res;
		}
		
		// highlight the first line number
		res.add(new TextAttribute(0, numberMatcher.end(), this.line));
		
		// and all other numbers we can find
		while (numberMatcher.find(numberMatcher.end())) {
			res.add(new TextAttribute(numberMatcher.start(), 
					numberMatcher.end()-numberMatcher.start(), this.number));
		}
		
		// keywords (commands)
		Matcher keywordMatcher = keywordRegex.matcher(line);	
		
		while (keywordMatcher.find()) {
			res.add(new TextAttribute(keywordMatcher.start(),
					keywordMatcher.end()-keywordMatcher.start(), this.keyword));
		}
		
		// strings
		Matcher stringMatcher = stringRegex.matcher(line);
		
		while (stringMatcher.find()) {
			res.add(new TextAttribute(stringMatcher.start(),
					stringMatcher.end()-stringMatcher.start(), this.string));
			
			// since these entries are processed later, they will overwrite any keywords/numbers
		}
		
		// remarks
		Matcher remMatcher = remRegex.matcher(line);
		
		while (remMatcher.find()) {
			res.add(new TextAttribute(remMatcher.start(2),
					remMatcher.end(2)-remMatcher.start(2), this.comment));
		}
		
		// open strings
		Matcher openString = openStringRegex.matcher(line);
		
		if (openString.matches()) {
			res.add(new TextAttribute(openString.start(2),
					openString.end(2)-openString.start(2), this.error));
		}
		
		return res;
		
	}
}
