package gui.editor;

import gui.widgets.ModificationListener;

import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import model.PData;

/**
 * A DocumentListener that will update the modified flag of a PData object whenever the document is changed.
 *
 * The flag has to be reset by a saver IOAction.
 *
 */
public class PDataUpdater implements DocumentListener {

	private PData d;
	private ModificationListener m;
		
	public PDataUpdater(PData d) {
		this.d = d;
	}
	
	public PDataUpdater(PData d, ModificationListener m) {
		this.d = d;
		this.m = m;
	}
	
	private void setModified() {
		if (!d.isModified()) {
			d.setModified(true);
			m.modified();
		}
	}
	
	@Override
	public void changedUpdate(DocumentEvent e) {
		setModified();
	}
	
	@Override
	public void insertUpdate(DocumentEvent e) {
		setModified();
	}
	
	@Override
	public void removeUpdate(DocumentEvent e) {
		setModified();
	}
}
