package gui.editor;

import java.text.FieldPosition;
import java.text.NumberFormat;
import java.text.ParsePosition;

public class RadixFormat extends NumberFormat {
	private static final long serialVersionUID = 4320952429209379869L;
	
	private int radix;
	private long max = -1;
	private boolean allowDouble; // TODO use parseIntegerOnly
	
	public RadixFormat(int radix, long max, boolean allowDouble) {
		this.radix = radix;
		this.max = max;
		this.allowDouble = allowDouble;
	}
	
	@Override
	public StringBuffer format(double number, StringBuffer toAppendTo,
			FieldPosition pos) {
		toAppendTo.append(number);
		return toAppendTo;
	}

	@Override
	public StringBuffer format(long number, StringBuffer toAppendTo,
			FieldPosition pos) {
	
		if (radix == 16)
			toAppendTo.append("0x");
		
		toAppendTo.append(Long.toString(number, radix));
		return toAppendTo;
	}

	@Override
	public Number parse(String source, ParsePosition parsePosition) {
		Number result = null;
		try {
			if (radix == 16 && source.startsWith("0x"))
				source = source.substring(2);
			
			if (!allowDouble)
				result = Long.valueOf(source, radix);
			else
				throw new NumberFormatException(); // we want doubles!
		} catch (NumberFormatException e) {
			try {
				if (!allowDouble)
					throw new NumberFormatException();
				
				result = Double.valueOf(source);
			} catch (NumberFormatException ex) {
				parsePosition.setErrorIndex(0);
				return null;
			}
		}
		
		if (max > 0 && result.doubleValue() > max) {
			parsePosition.setErrorIndex(0);
			return null;
		}
		
		parsePosition.setIndex(source.length());
		return result;
	}

}
