package gui.editor;

import java.util.List;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.Element;
import javax.swing.text.SimpleAttributeSet;

public abstract class SyntaxDocument extends DefaultStyledDocument {
	private static final long serialVersionUID = -7183102753951502927L;
	
	SimpleAttributeSet defaultAttributes = new SimpleAttributeSet();
	
	public SyntaxDocument() {
	}
	
	@Override
	public void insertString(int offs, String str, AttributeSet a)
			throws BadLocationException {
		// insert
		super.insertString(offs, str, a);

		// ugly hack (what a surprise) to highlight every line added
		int startOffset = offs;
		
		for (int i = 0; i < str.split("\n").length; i++) {
			Element e = getParagraphElement(startOffset);
			int pos = e.getStartOffset();
			int len = e.getEndOffset()-pos;
			String line = getText(pos, len);
			
			highlightLine(pos, len, line);
			
			startOffset = pos+len;
		}
		
	}
	
	@Override
	public void remove(int offs, int len) throws BadLocationException {
		super.remove(offs, len);
		
		// highlight (new) line at offset
		Element e = getParagraphElement(offs);
		int pos = e.getStartOffset();
		int elemLen = e.getEndOffset()-pos;
		
		String line = getText(pos, elemLen);
		
		highlightLine(pos, elemLen, line);
	}
	
	@Override
	public void replace(int offset, int length, String text, AttributeSet attrs)
			throws BadLocationException {
		// make sure that we first delete, then insert
		remove(offset, length);
		insertString(offset, text, attrs);
	}
	
	protected static class TextAttribute {
		private int start;
		private int len;
		private AttributeSet attrs;
		
		public TextAttribute(int start, int len, AttributeSet attrs) {
			this.start = start;
			this.len = len;
			this.attrs = attrs;
		}
	}
	
	/**
	 * Highlight the line
	 * @param line
	 * @return A list of (length, attribute) or (length, null) TextAttributes, splitting the line in
	 * segments to highlight.
	 */
	protected abstract List<TextAttribute> highlight(String line);
	
	protected void highlightLine(int pos, int len, String line) throws BadLocationException {
		setCharacterAttributes(pos, len, defaultAttributes, true);
	
		for (TextAttribute a: highlight(line)) {
			if (a.attrs != null)
				setCharacterAttributes(pos+a.start, a.len, a.attrs, true);
		}
			
	}
}
