package gui.widgets;

import gui.EasyLayout;
import gui.PDataEditor;
import gui.editor.ArrayTable;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.FieldPosition;
import java.text.Format;
import java.text.ParsePosition;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import model.PArray;
import model.PData;

public class ArrayEditor extends JPanel implements PDataEditor, ActionListener, TableModelListener {
	private static final long serialVersionUID = 8506046541319240251L;
	
	private PArray data;
	private ModificationListener modificationListener;
	
	private ArrayList<Integer> extraDimensions = new ArrayList<Integer>();
	private JTextField dimensionField;
	
	private JButton insertRowBefore;
	private JButton insertRowAfter;
	private JButton deleteRows;

	private JButton appendColumn;
	private JButton deleteColumns;
	
	private JComboBox layoutMode;
	
	private ArrayList<ArrayTable> tables = new ArrayList<ArrayTable>();
	private ArrayTable currentTable = null;
	
	private JButton applyLayout;
	
	private JScrollPane scrollPane;
	
	public ArrayEditor() {
		init();
	}
	
	private void init() {
		tables.add(new ArrayTable.NumberArrayTable());
		tables.add(new ArrayTable.Number2DArrayTable());
		tables.add(new ArrayTable.StringTable());
		tables.add(new ArrayTable.RawTable());
				
		setLayout(new BorderLayout());
		
		JPanel buttonPane = new JPanel(new EasyLayout());
		
		insertRowBefore = new JButton("Insert row before");
		insertRowBefore.setActionCommand("insertRowBefore");
		insertRowBefore.addActionListener(this);
		buttonPane.add(insertRowBefore, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		
		insertRowAfter = new JButton("Insert row after");
		insertRowAfter.setActionCommand("insertRowAfter");
		insertRowAfter.addActionListener(this);
		buttonPane.add(insertRowAfter, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		
		deleteRows = new JButton("Delete selected rows");
		deleteRows.setActionCommand("deleteRows");
		deleteRows.addActionListener(this);
		buttonPane.add(deleteRows, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		
		appendColumn = new JButton("Append column");
		appendColumn.setActionCommand("appendColumn");
		appendColumn.addActionListener(this);
		buttonPane.add(appendColumn, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);

		deleteColumns = new JButton("Delete selected columns");
		deleteColumns.setActionCommand("deleteColumns");
		deleteColumns.addActionListener(this);
		// buttonPane.add(deleteColumns, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL); TODO !!! (this is hard in java)
		
		buttonPane.add(new JSeparator(), EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		
		buttonPane.add(new JLabel("Dimension: "), EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		buttonPane.add(new JLabel("N x "));
				
		dimensionField = new JFormattedTextField(new Format() {
			private static final long serialVersionUID = -3114760524562762258L;

			@Override
			public StringBuffer format(Object obj, StringBuffer toAppendTo,
					FieldPosition pos) {
				toAppendTo.append(obj);
				return toAppendTo;
			}

			@Override
			public Object parseObject(String source, ParsePosition pos) {
				if (source.replaceAll(" ", "").matches("([0-9]+x)*([0-9]+)?")) {
					pos.setIndex(source.length());
					return source;
				}
				else {
					pos.setErrorIndex(0);
					return null;
				}
			}
			
		});
		dimensionField.setPreferredSize(new Dimension(40, dimensionField.getPreferredSize().height));
		
		
		buttonPane.add(dimensionField);
				
		buttonPane.add(new JLabel("Layout: "), EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		
		layoutMode = new JComboBox();
		
		for (ArrayTable t: tables) {
			layoutMode.addItem(t.getDescription());
		}
		
		layoutMode.setSelectedIndex(0);
		layoutMode.setPreferredSize(new Dimension(96, layoutMode.getPreferredSize().height));
		
		buttonPane.add(layoutMode);
		
		applyLayout = new JButton("Apply dimension & layout");
		applyLayout.setActionCommand("applyLayout");
		applyLayout.addActionListener(this);
		buttonPane.add(applyLayout, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		
		add(buttonPane, BorderLayout.EAST);
		
		// all set, now update the layout
		updateLayout();
		
		for (ArrayTable t: tables) {
			t.setModelListener(this);
		}
		
		/*
		addressTable = new JTable(model);
		addressTable.setDefaultEditor(Object.class, new DefaultCellEditor(new JTextField()) {
			@Override
			public boolean stopCellEditing() {
				String val = getCellEditorValue().toString();
				return val.matches("[0-9a-fA-F]{0,8}") && super.stopCellEditing();
			}
		});
		
		stringModel = new DefaultTableModel();
		stringModel.addColumn("Address (hexadecimal)");
		stringModel.addColumn("Length (decimal)");
		
		stringTable = new JTable(stringModel);
		stringTable.getColumnModel().getColumn(0).setCellEditor(new DefaultCellEditor(new JTextField()) {
			@Override
			public boolean stopCellEditing() {
				String val = getCellEditorValue().toString();
				return val.matches("[0-9a-fA-F]{0,4}") && super.stopCellEditing();
			}
		});
		
		stringTable.getColumnModel().getColumn(1).setCellEditor(new DefaultCellEditor(new JTextField()) {
			@Override
			public boolean stopCellEditing() {
				String val = getCellEditorValue().toString();
				int v;
				try {
					v = Integer.parseInt(val);
				} catch (NumberFormatException e) {
					v = -1;
				}
				return v >= 0 && v < 256 && super.stopCellEditing();
			}
		});
		*/
		/*
		JButton insertBefore = new JButton("Insert row before");
		insertBefore.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				
				int row = getSelectedTable().getSelectedRow();
				if (row == -1)
					row = 0;

				model.insertRow(row, new Object[]{""});
				stringModel.insertRow(row, new Object[]{"", ""});
			}
		});
		buttonPane.add(insertBefore, EasyLayout.NEW_ROW);
		
		JButton insertAfter = new JButton("Insert row after");
		insertAfter.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				int row = getSelectedTable().getSelectedRow()+1;
				if (row == 0)
					row = getSelectedTable().getRowCount();
				
				model.insertRow(row, new Object[]{""});
				stringModel.insertRow(row, new Object[]{"", ""});
			}
		});
		buttonPane.add(insertAfter);
		
		JButton deleteRow = new JButton("Delete selected");
		deleteRow.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				int rowOffset = 0;
				for (int row: getSelectedTable().getSelectedRows()) {
					model.removeRow(row-rowOffset);
					stringModel.removeRow(row-rowOffset);
					rowOffset++;
				}
			}
		});
		buttonPane.add(deleteRow);
		
		// add change listeners to both models
*/
	}
	
	private void updateLayout() {
		ArrayList<Integer> newDimensions = new ArrayList<Integer>();
		
		if (dimensionField.getText().length() > 0)
			for (String s: dimensionField.getText().split("x")) {
				s = s.trim();
				newDimensions.add(Integer.valueOf(s));
			}
		
		if (tables.get(layoutMode.getSelectedIndex()) instanceof ArrayTable.Number2DArrayTable && newDimensions.size() != 1) {
			JOptionPane.showMessageDialog(this, "This layout will only work for 2D arrays");
			return;
		}
		
		ArrayList<Long> arr = new ArrayList<Long>();
		// remove old table if necessary
		if (currentTable != null) {
			arr = currentTable.getArray();

			this.remove(scrollPane);
		}
		
		extraDimensions = newDimensions;
		
		currentTable = tables.get(layoutMode.getSelectedIndex());
		currentTable.setArray(arr, extraDimensions);
	
	    scrollPane = new JScrollPane(currentTable);
	    currentTable.setFillsViewportHeight(true);
		this.add(scrollPane);

		boolean columnsEnabled = currentTable.usesColumns();
		
		appendColumn.setEnabled(columnsEnabled);
		deleteColumns.setEnabled(columnsEnabled);
		
		validate();
	}
	
	@Override
	public void setData(PData data) {
		this.data = (PArray)data;
		
		this.extraDimensions = this.data.getExtraDimensions();
		updateDimensionField();
		
		currentTable.setArray(this.data.getArray(), extraDimensions);
	}

	@Override
	public void syncData() {
		ArrayList<Long> arrData = data.getArray();
		arrData.clear();
		
		data.getExtraDimensions().clear();
		data.getExtraDimensions().addAll(extraDimensions);
		arrData.addAll(currentTable.getArray());
		
		data.setNumber(
				currentTable instanceof ArrayTable.NumberArrayTable || 
				currentTable instanceof ArrayTable.Number2DArrayTable);
	}
	
	@Override
	public boolean useScrollbars() {
		return false;
	}
	
	@Override
	public void setModificationListener(ModificationListener m) {
		modificationListener = m;
	}
	
	// proxy this
	@Override
	public void tableChanged(TableModelEvent e) {
			if (modificationListener != null && !this.data.isModified()) {
				this.data.setModified(true);
				modificationListener.modified();
			}
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		// assert currentTable non-null
		if (e.getSource().equals(applyLayout))
			updateLayout();
		else if (e.getSource().equals(insertRowBefore))
			currentTable.insertRow(true);
		else if (e.getSource().equals(insertRowAfter))
			currentTable.insertRow(false);
		else if (e.getSource().equals(deleteRows))
			currentTable.deleteRows();
		else if (e.getSource().equals(appendColumn)) {
			currentTable.appendColumn();
			// assert successful
			extraDimensions.set(0, extraDimensions.get(0)+1);
			
			updateDimensionField();
		}
		else if (e.getSource().equals(deleteColumns)) {
			extraDimensions.set(0, extraDimensions.get(0)-currentTable.getSelectedColumnCount());
			currentTable.deleteColumns();
		}
	}

	private void updateDimensionField() {
		StringBuffer b = new StringBuffer();
		for (int d: extraDimensions)
			b.append(d+"x");
		
		if (b.length() != 0)
			b.deleteCharAt(b.length()-1);
		
		dimensionField.setText(b.toString());
	}
	
	// TODO: dimension field formatting, general table formatting, dimension parsing, column operations - remember to update dimension
}
