package gui.widgets;

import gui.PDataEditor;
import gui.editor.BasicDocument;
import gui.editor.PDataUpdater;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.AbstractAction;
import javax.swing.JPopupMenu;
import javax.swing.JTextPane;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.event.DocumentEvent;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.AbstractDocument;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;
import javax.swing.undo.UndoableEdit;

import model.PBasicProgram;
import model.PData;

public class BasicEditor extends JTextPane implements PDataEditor {
	private static final long serialVersionUID = 3990983001573576658L;

	private UndoManager undo = new UndoManager();
	private PBasicProgram program;
	private JPopupMenu popup;
	private PDataUpdater modifiedUpdater;
	
	
	public BasicEditor() {
		setBackground(UIManager.getDefaults().getColor("TextPane.background"));
		setFont(Font.decode(Font.MONOSPACED));
		setDocument(new BasicDocument());

		addUndoHooks();
		addPopup();
	}
	
	private class SignificanceWrapper implements UndoableEdit {

		private UndoableEdit proxy;
		private boolean significant;
		
		public SignificanceWrapper(UndoableEdit proxy, boolean significant) {
			this.proxy = proxy;
			this.significant = significant;
		}
		
		@Override
		public boolean addEdit(UndoableEdit anEdit) {
			return proxy.addEdit(anEdit);
		}

		@Override
		public boolean canRedo() {
			return proxy.canRedo();
		}

		@Override
		public boolean canUndo() {
			return proxy.canUndo();
		}

		@Override
		public void die() {
			proxy.die();
		}

		@Override
		public String getPresentationName() {
			return proxy.getPresentationName();
		}

		@Override
		public String getRedoPresentationName() {
			return proxy.getRedoPresentationName();
		}

		@Override
		public String getUndoPresentationName() {
			return proxy.getUndoPresentationName();
		}

		@Override
		public boolean isSignificant() {
			return significant;
		}

		@Override
		public void redo() throws CannotRedoException {
			proxy.redo();
		}

		@Override
		public boolean replaceEdit(UndoableEdit anEdit) {
			return proxy.replaceEdit(anEdit);
		}

		@Override
		public void undo() throws CannotUndoException {
			proxy.undo();
		}
		
	}
	
	private void addUndoHooks() {
	    getDocument().addUndoableEditListener(new UndoableEditListener() {
	    	@Override
	        public void undoableEditHappened(UndoableEditEvent e) {
	    		// XXX this is one hell of a hack!!!!
	    		undo.addEdit(new SignificanceWrapper(e.getEdit(), (e.getEdit() instanceof AbstractDocument.DefaultDocumentEvent)?
	    				(((AbstractDocument.DefaultDocumentEvent)e.getEdit()).getType() != DocumentEvent.EventType.CHANGE):true));
	        }
	    });
	    
	    getActionMap().put("undo",
	        new AbstractAction("Undo") {
				private static final long serialVersionUID = 4730393484390514941L;

				public void actionPerformed(ActionEvent evt) {
	                try {
	                    if (undo.canUndo()) {
	                        undo.undo();
	                    }
	                } catch (CannotUndoException e) {
	                }
	            }
	       });
	    
	    getInputMap().put(KeyStroke.getKeyStroke("control Z"), "undo");
	    
	    getActionMap().put("redo",
	        new AbstractAction("Redo") {
				private static final long serialVersionUID = 6710119039950190635L;

				public void actionPerformed(ActionEvent evt) {
	                try {
	                    if (undo.canRedo()) {
	                        undo.redo();
	                    }
	                } catch (CannotRedoException e) {
	                }
	            }
	        });
	    
	    getInputMap().put(KeyStroke.getKeyStroke("control Y"), "redo");
	}

	private void numberLines() {
		ArrayList<String> pureLines = new ArrayList<String>();
		ArrayList<Integer> lineNumbers = new ArrayList<Integer>();
		
		for (String line: getText().split("\n")) {
			if (line.isEmpty())
				continue;
			
			if (line.matches("^[0-9]+.*")) {
				String lineStr = line.replaceAll("^([0-9]+).*$", "$1");
				line = line.substring(lineStr.length()).trim();
				lineNumbers.add(Integer.parseInt(lineStr));
			}
			else
				lineNumbers.add(null);
			
			pureLines.add(line);
		}
		
		StringBuffer newLines = new StringBuffer();
		
		for (int i = 0; i < lineNumbers.size(); i++) {

			if (lineNumbers.get(i) == null) {
				int lineNum;
				int prevLine = i==0?0:lineNumbers.get(i-1);
				int nextLine = 0;
				int distance = 1;
				
				while (i+distance < lineNumbers.size()) {
					if (lineNumbers.get(i+distance) != null) {
						nextLine = lineNumbers.get(i+distance);
						break;
					}
					distance++;
				}
				
				if (i+distance == lineNumbers.size())
					lineNum = prevLine + 10;
				else {
					// there are "distance" unnumbered lines between prevLine and nextLine.
					// so interpolate
					//System.out.println("interpolating line between "+prevLine+" and "+nextLine+" with distance "+distance);
					lineNum = prevLine + (nextLine-prevLine)/(distance+1);
				}
				
				lineNumbers.set(i, lineNum);
			}

			newLines.append(lineNumbers.get(i)+" "+pureLines.get(i)+"\n");
		}
		
		newLines.deleteCharAt(newLines.length()-1); // last newline
		setText(newLines.toString());
	}
	
	private void addPopup() {
		popup = new JPopupMenu();
		popup.add("Re-number lines").addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				numberLines();
			}
		});
		
		addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				mouseReleased(e);
			}
			
			@Override
			public void mouseReleased(MouseEvent e) {
				if (e.isPopupTrigger())
					popup.show(e.getComponent(), e.getX(), e.getY());
			}
		});
	}
	
	@Override
	public void setData(PData data) {
		program = (PBasicProgram)data;

		try {
			getDocument().insertString(0, program.getProgramString(), new SimpleAttributeSet());
			
			if (modifiedUpdater != null)
				getDocument().removeDocumentListener(modifiedUpdater);
			getDocument().addDocumentListener(modifiedUpdater = new PDataUpdater(data, modificationListener));
		} catch (Exception e) {
			// should not occur
		}
	}
	
	@Override
	public void syncData() {
		try {
			program.setProgramString(getDocument().getText(0, getDocument().getLength()));
		} catch (Exception e) {
			// shouldn't happen
		}
	}
	
	
	@Override
	public boolean useScrollbars() {
		return true;
	}
	
	private ModificationListener modificationListener;
	
	@Override
	public void setModificationListener(ModificationListener m) {
		modificationListener = m;
	}
}
