package gui.widgets;

import gui.PDataEditor;
import gui.editor.HexDocument;
import gui.editor.PDataUpdater;

import java.awt.Font;

import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.text.BadLocationException;

import model.PData;
import model.PRawData;

public class HexEditor extends JSplitPane implements PDataEditor, CaretListener {
	private static final long serialVersionUID = -1257391058728225031L;

	private JTextArea asciiArea;
	private JTextArea hexArea;
	private PData data;
	
	private PDataUpdater modifiedUpdater;
	
	public HexEditor() {
		
		init();
	}
	
	private void init() {
		setOrientation(HORIZONTAL_SPLIT);
		HexDocument d = new HexDocument();
		
		asciiArea = new JTextArea(d.getAsciiDocument(), "", 5, 20);
		asciiArea.setLineWrap(true);
		asciiArea.setEditable(false);
		asciiArea.setFont(new Font(Font.MONOSPACED, Font.PLAIN, 12));
		
		add(new JScrollPane(asciiArea), JSplitPane.RIGHT);
		
		hexArea = new JTextArea(d, "", 5, 60);
		hexArea.setLineWrap(true);
		hexArea.setWrapStyleWord(true);
		hexArea.setFont(new Font(Font.MONOSPACED, Font.PLAIN, 12));
		add(new JScrollPane(hexArea), JSplitPane.LEFT);
		
		hexArea.addCaretListener(this);
	}
	
	protected void updateCaretSelection(int dot) {
		try {
			// get position in ascii text
			char[] text = hexArea.getText(0, dot).toCharArray();
			
			int pos = 0;
			for (char c: text) {
				 if (c != ' ' && c != '\n')
					 pos++;
			}
			
			// scroll to / select in ascii area
			// remember to compensate for the newlines
			asciiArea.requestFocus(true);
			//asciiArea.setCaretPosition(pos/2);
			asciiArea.select(pos/2+(pos/40), pos/2+(pos/40)+1);
			hexArea.requestFocus();
			asciiArea.validate();
			
		} catch (BadLocationException e1) {
			// shouldn't happen
		} catch (IllegalArgumentException e1) {
			// ignore
			//System.out.println(e1.getMessage());
		}		
	}
	
	@Override
	public void caretUpdate(CaretEvent e) {
		updateCaretSelection(e.getDot());
	}

	@Override
	public void setData(PData data) {
		this.data = data; 
		
		short[] encodedData;
		
		if (data instanceof PRawData)
			encodedData = ((PRawData)data).getData();
		else
			encodedData = data.encode(); 
		
		StringBuffer sb = new StringBuffer();
		
		if (encodedData == null)
			return;
		
		int cnt = 0;
		for (short s: encodedData) {
			sb.append((s<16?"0":"")+Integer.toHexString(s));
			sb.append(cnt % 20 == 19?'\n':' ');
			cnt++;
		}
		
		hexArea.insert(sb.toString(), 0);
		
		if (modifiedUpdater != null)
			hexArea.getDocument().removeDocumentListener(modifiedUpdater);
		hexArea.getDocument().addDocumentListener(modifiedUpdater = new PDataUpdater(data, modificationListener));
		
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				setDividerLocation(0.8);
				validate();
			}
		});

	}
	
	@Override
	public void syncData() {
		try {
			String bytes = hexArea.getDocument().getText(0, hexArea.getDocument().getLength()).replaceAll("[^0-9a-fA-F]", "");
			short[] shorts = new short[bytes.length()/2];
			
			for (int i = 0; i < shorts.length; i++)
				shorts[i] = Short.valueOf(bytes.substring(i*2, i*2+2), 16);
			
			if (data instanceof PRawData)
				((PRawData)data).setData(shorts);
			else
				data.decode(shorts);
			
		} catch (BadLocationException e) {
			// shouldn't happen
		}
	}
	
	@Override
	public boolean useScrollbars() {
		return false;
	}
	
	private ModificationListener modificationListener;
	
	@Override
	public void setModificationListener(ModificationListener m) {
		modificationListener = m;
	}
}
