package gui.widgets;

import gui.EasyLayout;
import gui.PDataEditor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;
import javax.swing.SpinnerNumberModel;
import javax.swing.UIManager;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import model.PData;
import model.PImage;

public class ImageEditor extends JPanel implements PDataEditor, TableModelListener {
	private static final long serialVersionUID = -471604213466481965L;

	private static final int COLS = 40;
	private static final int ROWS = 23;
	
	private DefaultTableModel model; 
	private JTable table;
	private PImage img;
	
	private JSpinner charValue;
	private JComboBox foreground;
	private JCheckBox blink;
	
	private JCheckBox graphicsMode;
	
	private JComboBox background;
	
	private JCheckBox reverse;
	private JCheckBox doubleWidth;
	private JCheckBox doubleHeight;
	
	private JButton changeCustomChars;
	private JTextArea customCharArea;
	private JPanel customCharPanel;
	
	private CharacterComponent comp = new CharacterComponent();
	
	private boolean updatingFields; // EWWWWW!!!!!!
	private boolean reverseSelected = true;
	private ArrayList<Integer> doubleHeightStartRows = new ArrayList<Integer>();
	
	private static Color[] machineColors = new Color[] {
		Color.BLACK,
		Color.RED,
		Color.GREEN,
		Color.YELLOW,
		Color.BLUE,
		Color.MAGENTA,
		Color.CYAN,
		Color.WHITE
	};
	
	private static String[] colorNames = new String[] {
		"Black", "Red", "Green", "Yellow", "Blue", "Magenta", "Cyan", "White"
	};
	
	private static int getColorIndex(Color c) {
		for (int i = 0; i < machineColors.length; i++)
			if (machineColors[i].equals(c))
				return i;
		
		return 0;
	}
	
	public ImageEditor() {
		init();
	}
	
	@SuppressWarnings("serial")
	void init() {
		model = new DefaultTableModel();
		
		for (int x = 0; x < COLS; x++) {
			model.addColumn(/*(x<10?"0":"")+Integer.toString(x)*/null);
		}
		for (int y = 0; y < ROWS; y++) {
			ColoredChar[] row = new ColoredChar[COLS];
			row[0] = new ColoredChar((char)0x80, 0x66);
			for (int i = 1; i < COLS; i++)
				row[i] = new ColoredChar(' ', 0x80);
				
			model.addRow(row);
		}
		
		setLayout(new BorderLayout());
		
		table = new JTable(model);
		table.setTableHeader(null);
		table.setShowGrid(false);
		table.setRowMargin(0);
		table.getColumnModel().setColumnMargin(0);
		table.setCellSelectionEnabled(true);
		//table.setFillsViewportHeight(true);
		
		//table.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		
		table.setDefaultRenderer(Object.class, new TableCellRenderer() {
		
			// this will set the color to the foreground of the last character >= 127 on the row (or to its background, if in graphics mode)
			private Color backgroundForRow(int row, int col) {
				Color ret = machineColors[0];
				for (int i = 0; i < col; i++) {
					ColoredChar c = (ColoredChar)table.getValueAt(row, i);
					if (c != null && (c.c > 127 || c.graphicsMode))
						// TODO factor out
						ret = c.graphicsMode?c.bg:machineColors[(c.doubleHeight?1:0) + (c.doubleWidth?2:0) + (c.reverse?4:0)];
				}
				
				return ret;
			}

			private boolean startDoubleWidth(int row, int column) {
				// search row backwards for the first double width char, this will start it
				ColoredChar ch = (ColoredChar)table.getValueAt(row, column);
				
				if (ch.graphicsMode || !ch.doubleWidth)
					return true;

				boolean mod = true;
				
				for (int x = column-1; x >= 0; x--) {
					ColoredChar c = (ColoredChar)table.getValueAt(row, x);
					if (c == null || !c.graphicsMode && !c.doubleWidth || c.graphicsMode) // found first non-double-width char
						break;
					
					mod = !mod;
				}
				
				return mod;
			}


			private boolean startDoubleHeight(int row, int column) {
				// search row backwards for the first double width char, this will start it
				ColoredChar ch = (ColoredChar)table.getValueAt(row, column);
				
				if (ch.graphicsMode || !ch.doubleHeight)
					return true;

				// we use the cached doubleHeightStartRows to determine this for speed
				int lastRow = 0;
				
				for (int startRow: doubleHeightStartRows) {
					if (startRow > row)
						break;
					lastRow = startRow;
				}
				
				return Math.abs(lastRow - row) % 2 == 0;
			}
			
			@Override
			public Component getTableCellRendererComponent(JTable table,
					Object value, boolean isSelected, boolean hasFocus,
					int row, int column) {

				comp.setFocus(hasFocus);
				comp.setSelected(isSelected && reverseSelected);
				comp.setBackground(backgroundForRow(row, column));
				
				if (value == null) {
					comp.setChar(new ColoredChar(' ', 0x08));
				}
				else {
					
					boolean startDoubleHeight = startDoubleHeight(row, column);
					boolean startDoubleWidth = startDoubleWidth(row, column);
					
					comp.setChar((ColoredChar)value, startDoubleHeight, startDoubleWidth);
				}
				
				return comp;
			}

		});
		
		table.setDefaultEditor(Object.class, new DefaultCellEditor(new JTextField(1)) {
			private ColoredChar c;
			
			@Override
			public Component getTableCellEditorComponent(JTable table,
					Object value, boolean isSelected, int row, int column) {
				c = (ColoredChar)value;
				
				JTextField editor = 
					(JTextField)super.getTableCellEditorComponent(table, Character.toString(c.c), isSelected, row, column);
				
				editor.selectAll();
				return editor;
			}
			
			@Override
			public Object getCellEditorValue() {
				String val = super.getCellEditorValue().toString();
				c.c = val.isEmpty()?' ':val.charAt(val.length()-1);
				return c;
			}
			
		});
		
		table.getDefaultEditor(Object.class).addCellEditorListener(new CellEditorListener() {
			@Override
			public void editingStopped(ChangeEvent e) {
				charValue.setValue((int)((ColoredChar)(table.getValueAt(table.getSelectedRow(), table.getSelectedColumn()))).c);
			}
			
			@Override
			public void editingCanceled(ChangeEvent e) {
				// don't care
			}
		});
		
		JPanel centerPanel = new JPanel(new EasyLayout()); // XXX ugly
		centerPanel.add(new JLabel(" "), EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL+EasyLayout.VERTICAL_FILL);
		centerPanel.add(new JLabel(" "), EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		table.setPreferredSize(new Dimension(COLS*2*8, ROWS*2*10));
		table.setRowHeight(20);
		centerPanel.add(table);
		centerPanel.add(new JLabel(" "), EasyLayout.HORIZONTAL_FILL);
		centerPanel.add(new JLabel(" "), EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL+EasyLayout.VERTICAL_FILL);
		
		add(new JScrollPane(centerPanel), BorderLayout.CENTER);
		
		// panel for editing colors/entire selection
		JPanel rightPanel = new JPanel(new EasyLayout());
		
		rightPanel.add(new JLabel("Character code:"), EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		
		charValue = new JSpinner(new SpinnerNumberModel(0,0,255,1));
		
		rightPanel.add(charValue);
		
		blink = new JCheckBox("Blink");
		
		rightPanel.add(blink, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		
		foreground = new JComboBox(colorNames);
		background = new JComboBox(colorNames);

		foreground.setRenderer(new ColorRenderer());
		background.setRenderer(new ColorRenderer());
				
		int i = 0;
		for (Color c: machineColors) {
			BufferedImage im = new BufferedImage(12, 12, BufferedImage.TYPE_INT_RGB);
			Graphics g = im.getGraphics();
			g.setColor(c);
			g.fillRect(0, 0, im.getWidth(), im.getHeight());
			
			foreground.add(new JLabel(colorNames[i], new ImageIcon(im), JLabel.LEFT));
			background.add(new JLabel(colorNames[i], new ImageIcon(im), JLabel.LEFT));
			i++;
		}
		
		rightPanel.add(new JLabel("Foreground:"), EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		rightPanel.add(foreground);
		
		graphicsMode = new JCheckBox("Graphics mode");
		rightPanel.add(graphicsMode, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);

		rightPanel.add(new JLabel("Background:"), EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		rightPanel.add(background);
		
		doubleWidth = new JCheckBox("Double width");
		rightPanel.add(doubleWidth, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		
		doubleHeight = new JCheckBox("Double height");
		rightPanel.add(doubleHeight, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		
		reverse = new JCheckBox("Reverse");
		rightPanel.add(reverse, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		
		changeCustomChars = new JButton("Edit custom characters");
		rightPanel.add(changeCustomChars, EasyLayout.NEW_ROW+EasyLayout.HORIZONTAL_FILL);
		
		// listeners
		charValue.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				if (updatingFields)
					return;
				
				LinkedList<ColoredChar> sc = getSelectedChars();
				for (ColoredChar c: sc) {
					c.c = (char)((Integer)(charValue.getValue())).intValue();
				}
				updateSelectedChars(sc);
			}
		});
		
		foreground.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (updatingFields)
					return;
				
				LinkedList<ColoredChar> sc = getSelectedChars();
				for (ColoredChar c: sc) {
					c.fg = machineColors[foreground.getSelectedIndex()];
				}
				updateSelectedChars(sc);
			}
		});
		
		blink.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (updatingFields)
					return;
				
				LinkedList<ColoredChar> sc = getSelectedChars();
				for (ColoredChar c: sc) {
					c.blink = blink.isSelected();
				}
				updateSelectedChars(sc);
			}			
		});
		
		ActionListener graphicsListener = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (updatingFields)
					return;
				
				LinkedList<ColoredChar> sc = getSelectedChars();
				for (ColoredChar c: sc) {
					c.graphicsMode = graphicsMode.isSelected();
					if (c.graphicsMode)
						c.bg = machineColors[background.getSelectedIndex()];
					else {
						c.doubleHeight = doubleHeight.isSelected();
						c.doubleWidth = doubleWidth.isSelected();
						c.reverse = reverse.isSelected();
					}
				}
				updateSelectedChars(sc);
				updateDoubleHeightStartRow();
			}
		};
		
		graphicsMode.addActionListener(graphicsListener);
		graphicsMode.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (updatingFields)
					return;
				
				boolean g = graphicsMode.isSelected();
				background.setEnabled(g);
				doubleWidth.setEnabled(!g);
				doubleHeight.setEnabled(!g);
				reverse.setEnabled(!g);
			}			
		});
		
		background.addActionListener(graphicsListener);
		doubleHeight.addActionListener(graphicsListener);
		doubleWidth.addActionListener(graphicsListener);
		reverse.addActionListener(graphicsListener);
		
		ListSelectionListener changeListener = new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				
				if (e.getValueIsAdjusting()) {
					return; // ignore this
				}
				
				ColoredChar selectedItem = (ColoredChar)model.getValueAt(table.getSelectedRow(), table.getSelectedColumn());
				
				// search for foreground / background
				updatingFields = true;
				charValue.setValue((int)selectedItem.c);
				foreground.setSelectedIndex(getColorIndex(selectedItem.fg));
				blink.setSelected(selectedItem.blink);
				
				background.setSelectedIndex(getColorIndex(selectedItem.bg));
				doubleHeight.setSelected(selectedItem.doubleHeight);
				doubleWidth.setSelected(selectedItem.doubleWidth);
				reverse.setSelected(selectedItem.reverse);
				
				graphicsMode.setSelected(selectedItem.graphicsMode);
				
				boolean g = graphicsMode.isSelected();
				background.setEnabled(g);
				doubleWidth.setEnabled(!g);
				doubleHeight.setEnabled(!g);
				reverse.setEnabled(!g);
				updatingFields = false;
				
				validate();
			}
		};

		// JTable - now sporting more hacks than you'd like
		table.getColumnModel().getSelectionModel().addListSelectionListener(changeListener);
		table.getSelectionModel().addListSelectionListener(changeListener);
		
		
		changeCustomChars.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				// show an edit dialog
				if (getCustomChars())
					table.repaint();
			}
		});
		
		customCharArea = new JTextArea(10, 30);
		customCharPanel = new JPanel(new BorderLayout());
		customCharPanel.add(new JLabel("Paste SETEG/SETET lines here to set the respective custom characters."), BorderLayout.NORTH);
		customCharPanel.add(new JScrollPane(customCharArea));
		
		add(rightPanel, BorderLayout.EAST);
		
		table.getModel().addTableModelListener(this);
		
	}
	
	private LinkedList<ColoredChar> getSelectedChars() {
		LinkedList<ColoredChar> l = new LinkedList<ColoredChar>();
		
		for (int row: table.getSelectedRows()) {
			for (int col: table.getSelectedColumns()) {
				l.add((ColoredChar)model.getValueAt(row, col));
			}
		}		
		return l;
	}
	
	private void updateSelectedChars(LinkedList<ColoredChar> l) {
		for (int row: table.getSelectedRows()) {
			for (int col: table.getSelectedColumns()) {
				model.setValueAt(l.pop(), row, col);
			}
		}				
	}
	
	// remember to call this every time the first "double height" row may change 
	private void updateDoubleHeightStartRow() {
		doubleHeightStartRows.clear();
		int lastRow = -2; // last row where we encountered a double height character
		
		for (int row = 0; row < ROWS; row++) {
			for (int col = 0; col < COLS; col++) {
				ColoredChar coloredChar = (ColoredChar)model.getValueAt(row, col);
				
				if (coloredChar != null && !coloredChar.graphicsMode && coloredChar.c < 128 && coloredChar.doubleHeight) {
					if (lastRow != row - 1)
						doubleHeightStartRows.add(row); // if the last row wasn't double-height, add this row as a start row
					lastRow = row;
					break; // next row
				}
			}
		}
	}
	
	@Override
	public void setData(PData data) {
		img = (PImage)data;
		
		int[][] colors = img.getColors();
		char[][] chars = img.getChars();
		
		for (int row = 0; row < ROWS && row < chars.length; row++) {
			for (int col = 0; col < COLS && col < chars[row].length; col++) {
				ColoredChar coloredChar = new ColoredChar(chars[row][col], colors[row][col]);
				model.setValueAt(coloredChar, row, col);
			}
		}
		
		updateDoubleHeightStartRow();
	}
	
	@Override
	public void syncData() {
		int[][] colors = img.getColors();
		char[][] chars = img.getChars();
		
		for (int row = 0; row < ROWS && row < chars.length; row++) {
			for (int col = 0; col < COLS && col < chars[row].length; col++) {
				ColoredChar coloredChar = (ColoredChar)model.getValueAt(row, col);
				chars[row][col] = coloredChar.c;
				colors[row][col] = coloredChar.getFormat();
			}
		}
	}
	
	@Override
	public boolean useScrollbars() {
		return false;
	}
	
	public class ColoredChar {
		char c;
		Color fg;
		boolean blink;
		boolean graphicsMode;

		// graphics mode
		Color bg = machineColors[0];
		
		// text mode
		boolean doubleHeight;
		boolean doubleWidth;
		boolean reverse;
		
		ColoredChar() {
			// do nothing
		}
		
		ColoredChar(char c, int color) {
			this.c  = c;
			
			fg = machineColors[color&0x07];
			blink = (color&0x08) == 0;
			
			graphicsMode = (color&0x80) != 0;
			
			if (graphicsMode) {
				bg = machineColors[(color>>4)&0x07];
			}
			else {
				doubleHeight = (color&0x10) != 0;
				doubleWidth = (color&0x20) != 0;
				reverse = (color&0x40) != 0; 
			}
		}
		
		int getFormat() { // get the color byte
			if (graphicsMode)
				return 0x80|(getColorIndex(bg)<<4)|(blink?0x00:0x08)|(getColorIndex(fg));
			else
				return (reverse?0x40:0x00)|(doubleWidth?0x20:0x00)|(doubleHeight?0x10:0x00)|
				(blink?0x00:0x08)|(getColorIndex(fg));
		}
	}
	
	// color dropdown
	private class ColorRenderer implements ListCellRenderer {
		private JLabel l = new JLabel();
		private ArrayList<Icon> icons = new ArrayList<Icon>();	
		
		public ColorRenderer() {
			
			for (Color c: machineColors) {
				BufferedImage im = new BufferedImage(12, 12, BufferedImage.TYPE_INT_RGB);
				Graphics g = im.getGraphics();
				g.setColor(c);
				g.fillRect(0, 0, im.getWidth(), im.getHeight());
				
				icons.add(new ImageIcon(im));
			}
		}
		
		@Override
		public Component getListCellRendererComponent(JList list,
				Object value, int index, boolean isSelected,
				boolean cellHasFocus) {
			
			isSelected |= cellHasFocus; 
			boolean first = false;
			
			if (index == -1) {
				index = list.getSelectedIndex();
				first = true;
			}
			
			if (index == -1) {
				l.setIcon(null);
				l.setText("");
				return l;
			}
			
			l.setIcon(icons.get(index));
			l.setText(colorNames[index]);
			
			// TODO use other borders
			if (first) {
				l.setBackground(UIManager.getColor("text"));
				l.setForeground(UIManager.getColor("textText"));
			}
			else {
				l.setBackground(isSelected?UIManager.getColor("List.selectionBackground"):UIManager.getColor("List.background"));
				l.setForeground(isSelected?UIManager.getColor("List.selectionForeground"):UIManager.getColor("List.foreground"));
			}
			
			l.setBorder(first?BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.LOWERED):BorderFactory.createEmptyBorder());
			
			return l;
		}
	}
	
	
	private ModificationListener modificationListener;
	
	@Override
	public void setModificationListener(ModificationListener m) {
		modificationListener = m;
	}

	@Override
	public void tableChanged(TableModelEvent e) {
		if (modificationListener != null && !img.isModified()) {
			modificationListener.modified();
			img.setModified(true);
		}
	}
	
	private boolean blinkingStarted = false;
	private Timer blinkingTimer = null;
	
	@Override
	public synchronized void setVisible(boolean flag) {
		super.setVisible(flag);
		if (flag && !blinkingStarted) {
			blinkingTimer = new Timer(true);
			
			// start the blinking thread
			blinkingTimer.scheduleAtFixedRate(new TimerTask() {
				@Override
				public void run() {
					// blink!
					comp.setBlink(!comp.isBlink());
					table.repaint();
				}
			}, 0, 1000);
			blinkingStarted = true;
		}
		else if (!flag && blinkingStarted) {
			blinkingTimer.cancel();
			blinkingStarted = false;
		}
	}
	
	private boolean getCustomChars() {
		if (JOptionPane.showConfirmDialog(this, customCharPanel, "Custom character editor", 
				JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE) == JOptionPane.OK_OPTION) {
			comp.parseChars(customCharArea.getText());
			return true;
		}
		
		return false;
	}
	
	// messy method exposing stuff for the migrator
	public Dimension getTableDimensions() {
		return table.getPreferredSize();
	}
	
	public synchronized void paintTable(Graphics g, boolean blink) {
		boolean originalBlink = comp.isBlink();
		reverseSelected = false;
		
		comp.setBlink(blink);
		table.paintAll(g);
		
		comp.setBlink(originalBlink);
		reverseSelected = true;
	}
}
