package model;

import gui.CassetteLogger;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.text.DecimalFormat;
import java.text.NumberFormat;


public class PBasicProgram extends PData {

	static String[] bascom =
	{
		"END","FOR","NEXT","DATA","INPUT","DIM","READ","LET",
		"GOTO","RUN","IF","RESTORE","GOSUB","RETURN","REM","STOP",
		"ON","LPRINT","DEF","POKE","PRINT","CONT","LIST","LLIST",
		"CLEAR","CLOAD","CSAVE","TX","GR","SCREEN","LINE","DISPLAY",
		"STORE","INIT","CURSORX","CURSORY","SCROLL","PAGE","BRIGHT","SOUND",
		"DELIM","SETET","SETEG","ET","EG","NEW","TAB(","TO",
		"FN","SPC(","THEN","NOT","STEP","+","-","*",
		"/","^","AND","OR",">","=","<","SGN",
		"INT","ABS","USR","FRE","LPOS","POS","SQR","RND",
		"LOG","EXP","COS","SIN","TAN","ATN","PEEK","LEN",
		"STR$","VAL","ASC","STICKX","STICKY","ACTION","KEY","CHR$",
		"LEFT$","RIGHT$","MID$"
	};

	protected String programString = "";

	public String toString() {
		return programString;
	}
	
	public String getProgramString() {
		return programString;
	}
	
	public void setProgramString(String programString) {
		this.programString = programString;
	}
	
	@Override
	public void fromInputStream(InputStream i) throws IOException {
		String d = decodeVerboseHeader(i);
		
		StringBuffer sb = new StringBuffer();
		
		if (d != null) {
			d = d.replaceAll("[\n\r]+$", "");
			if (d.length() > 0) 
				sb.append(d+"\n");
		}
		
		String line = null;
		BufferedReader in = new BufferedReader(new InputStreamReader(i));
		while ((line = in.readLine()) != null)
			sb.append(line+"\n");
		
		programString = sb.toString();
	}
		
	@Override
	public void toOutputStream(OutputStream o) throws IOException {
		encodeVerboseHeader(o);
		
		PrintWriter w = new PrintWriter(o);
		w.print(programString);
		w.flush();
	}
	
	@Override
	public short[] encode() {
		short[] BINData;    //binary data
		int BINDataUsage=0; //space used in BINData Array
		BINData = new short[17000]; //16K RAM + some file overhead space
		
//		FILE *fp, *fq;
		short c,c1; //,readstr[200],baslin[80],basnam[32];
		short[] baslin=new short[80];
		String readstr;
		int fp=0;
		int i=0,prevline,linenum,numlin,found,offs,strng,n,addr;

		numlin = 0;
		prevline = 0;
		addr = 35009;

		while (fp<programString.length() && (programString.charAt(fp) < '0' || programString.charAt(fp) > '9'))
			fp++; // skip ahead
		
		linenum=0;
		while (fp<programString.length() && programString.charAt(fp)>='0' && programString.charAt(fp)<='9') {
			linenum=linenum*10+(programString.charAt(fp++)-'0');
		}

		while (linenum > 0)
		{
		   numlin++;
		   if (linenum < prevline)
			   CassetteLogger.warn("Warning: line number "+linenum+" lower than previous line number "+prevline+".");
		   else prevline = linenum;

		   c = (short)(linenum % 256);
		   c1 = (short)(linenum / 256);
		   baslin[0]=c;
		   baslin[1]=c1;
		   n = 2;

		   readstr="";
		   while (fp<programString.length() && programString.charAt(fp)!='\n' && programString.charAt(fp)!='\r')
			   readstr=readstr+programString.charAt(fp++);
		   //fgets(readstr,199,fp);
		   if (readstr.length() > 35)
			   CassetteLogger.warn("Warning: line " + linenum + " is too long.");

		   offs = 1;
		   strng = 0;
		   
		   if (linenum==60)
			   strng=0;

		   while(offs<readstr.length() &&  readstr.charAt(offs)>=0x20)
		   {
			found = 0;
			if (readstr.charAt(offs) == '\"')
			   if (strng == 0) strng = 1; else strng = 0;
			if (strng == 0) for(i=0;i<bascom.length&&found==0;i++)
			{
			   if (readstr.length()>=offs+bascom[i].length() &&
					   readstr.substring(offs,offs+bascom[i].length()).toUpperCase().compareTo(bascom[i])==0) {
				   found=1;
				   break;
			   }			   
			}
			if (found == 1)
			{
			   c = (short)(i+128);
			   offs += bascom[i].length();
			   if (c == 0x8e) strng = 1;
			}
			else
			{
			   c = (short)readstr.charAt(offs);
			   switch (c) {
				case 0xA3: c = 0x03; break;
				case '$' : c = 0x04; break;
				case '#' : c = 0x06; break;
				case 0xA8: c = 0x7e; break;
				case '^' : c = 0x7f; break;
			   }
			   offs++;
			}
			baslin[n] = c;
			n++;
		   }
		   baslin[n] = 0;
		   addr += n + 3;
		   c = (short)(addr % 256);
		   c1 = (short)(addr / 256);
		   BINData[BINDataUsage++]=c;BINData[BINDataUsage++]=c1;		   
		   for (i=0;i<=n;i++) {BINData[BINDataUsage++]=baslin[i];}
		   
		   //fscanf(fp,"%d",&linenum);
		   linenum=0;
		   while (fp<programString.length() && (programString.charAt(fp)=='\n' || programString.charAt(fp)=='\r'))fp++;		   
		   while (fp<programString.length() && programString.charAt(fp)>='0' && programString.charAt(fp)<='9') {
			   linenum=linenum*10+(programString.charAt(fp++)-'0');
		   }

		}
		BINData[BINDataUsage++]=0;
		BINData[BINDataUsage++]=0;
				

		//if (ftell(fp) < tsiz - 8) CassetteLogger.warn("Warning: last line read: "+prevline);
		CassetteLogger.info(numlin+" lines read.");
		//CassetteLogger.info("BASIC program name: "+basnam);

		short[] binblock=new short[BINDataUsage]; 
		System.arraycopy(BINData,0,binblock,0, BINDataUsage);		
		CassetteLogger.info("encoded: " + BINDataUsage + " data bytes.");



			 int width=32;
			 int r;
			 int l=0;
			 int s=0;
			 String asc="";
			 String hex="";
			 String h="";
			 NumberFormat formatter = new DecimalFormat("00000");		 
			 
			for (r=0;r<BINDataUsage;r++) {
				h=Integer.toHexString(BINData[r]);
				if (h.length()==1) 
					hex=hex + "0" + h + " ";
				else
					hex=hex + h + " ";
				
				if (BINData[r]<32) 
					asc=asc+".";
				else
					asc=asc + (char)(BINData[r]);
				
				l++;
				
				if (l==width) {
					System.out.println(formatter.format(s) + ": " + hex + asc);
					s=s+l;
					l=0;
					hex="";
					asc="";
				}		
			}
			if (hex.length()!=0) {
				for (r=l;r<width;r++)
					hex=hex+"   ";
				System.out.println(formatter.format(s) + ": " + hex + asc);
			}	
		
		
		
		return binblock;
	}

	
	@Override
	public void decode(short[] data) {
		
		boolean firstline=true;
		int c,c1,j=0,fp;
		int filend=0;
		int nextaddr=0;
		int prevline=0;
		NumberFormat formatter = new DecimalFormat("00000");

		fp = 0;  // startoffset BASIC program
		c = 0;
		while((fp<data.length)&&(filend==0))
		{
			if (c==0x00)
			{
				if (fp != nextaddr)
					CassetteLogger.warn("Warning: address mismatch at " + fp + ", expected: " + nextaddr);
				c=data[fp++];c1=data[fp++];
				j=256*c1+c;
				if(j==0)filend=1;
				else
				{
				   nextaddr = (j - 0x88c1); // was + 1
				   c=data[fp++];c1=data[fp++];
				   j=256*c1+c;
				   if (firstline==false) { // System.out.print("\n");
					   programString = programString + "\n";				   
				   }
				   firstline=false;
				   if (j < prevline)
					   CassetteLogger.warn("Warning: line number " + j + " lower than previous line " + prevline);
				   else prevline = j;
				   //System.out.print(formatter.format(j) + " ");
				   programString = programString + formatter.format(j) + " ";
				}
			}
			else
			if (c==0x03) {c1=0xA3; //System.out.print((char)c1);
				programString = programString + (char)c1;
			}
			else
			if (c==0x04) // System.out.print("$");
				programString = programString + "$";
			else
			if (c==0x06) // System.out.print("#");
				programString = programString + "#";
			else
			if ((c < 0x20) || (c > 0xda))
				CassetteLogger.warn("Warning: Illegal byte " + c + " at address " + (fp-1));
			else
			if (c < 0x7e) // System.out.print((char)c);
				programString = programString + (char)c;
			else
			if (c==0x7e) {c1=0xA8; // System.out.print((char)c1);
					programString = programString + (char)c1;
				}
			else
			if (c==0x7f) // System.out.print("^");
				programString = programString + "^";
			//else System.out.print("<BAS>" + bascom[c-128] + "</BAS>");
			else //System.out.print(bascom[c-128]);
				programString = programString + bascom[c-128];
			
			if (fp < data.length)
				c=data[fp++];
			else
				filend=1;
		}		
	}
	
	@Override
	public String getDefaultExtension() {
		return "bas";
	}
	
	@Override
	public char getFormat() {
		return ' ';
	}
	
	@Override
	public int getAddress() { // fixed address (in case the header is corrupted)
		if (super.getAddress() == 0x0)
			return 0x88c1;
		else
			return super.getAddress();
	}

}
